@@ -27,47 +27,59 @@
 #include "GLibSource.h"
 #include "Variant.h"
 #include "DBusIndicators.h"
+#include "services/panel-service-private.h"
 
 namespace unity
 {
 namespace indicator
 {
-DECLARE_LOGGER(logger, "unity.indicator.dbus");
 
 namespace
 {
-const std::string SERVICE_NAME("com.canonical.Unity.Panel.Service");
-const std::string SERVICE_PATH("/com/canonical/Unity/Panel/Service");
-const std::string SERVICE_IFACE("com.canonical.Unity.Panel.Service");
+DECLARE_LOGGER(logger, "unity.indicator.dbus");
+
+inline bool verify_variant_type(GVariant* value, const gchar* type)
+{
+  if (!g_variant_is_of_type (value, G_VARIANT_TYPE(type)))
+  {
+    LOG_ERROR(logger) << "Got invalid variant type: '"
+                      << g_variant_get_type_string(value) << "' ('"
+                      << type << "' was expected)";
+    return false;
+  }
+
+  return true;
+}
 } // anonymous namespace
 
 
 /* Connects to the remote panel service (unity-panel-service) and translates
  * that into something that the panel can show */
-class DBusIndicators::Impl
+struct DBusIndicators::Impl
 {
-public:
   Impl(std::string const& dbus_name, DBusIndicators* owner);
 
   void CheckLocalService();
   void RequestSyncAll();
   void RequestSyncIndicator(std::string const& name);
-  void Sync(GVariant* args);
+  void Sync(GVariant* args, glib::Error const&);
   void SyncGeometries(std::string const& name, EntryLocationMap const& locations);
+  void ShowEntriesDropdown(Indicator::Entries const&, Entry::Ptr const&, unsigned xid, int x, int y);
+  void CloseActiveEntry();
 
   void OnConnected();
   void OnDisconnected();
 
   void OnReSync(GVariant* parameters);
+  void OnIconsPathChanged(GVariant* parameters);
   void OnEntryActivated(GVariant* parameters);
   void OnEntryActivatedRequest(GVariant* parameters);
   void OnEntryShowNowChanged(GVariant* parameters);
 
-  virtual void OnEntryScroll(std::string const& entry_id, int delta);
-  virtual void OnEntryShowMenu(std::string const& entry_id, unsigned int xid,
-                               int x, int y, unsigned int button);
-  virtual void OnEntrySecondaryActivate(std::string const& entry_id);
-  virtual void OnShowAppMenu(unsigned int xid, int x, int y);
+  void OnEntryScroll(std::string const& entry_id, int delta);
+  void OnEntryShowMenu(std::string const& entry_id, unsigned int xid, int x, int y, unsigned int button);
+  void OnEntrySecondaryActivate(std::string const& entry_id);
+  void OnShowAppMenu(unsigned int xid, int x, int y);
 
   DBusIndicators* owner_;
 
@@ -75,17 +87,19 @@ public:
   glib::Source::UniquePtr reconnect_timeout_;
   glib::Source::UniquePtr show_entry_idle_;
   glib::Source::UniquePtr show_appmenu_idle_;
-  std::map<std::string, EntryLocationMap> cached_locations_;
+  std::vector<std::string> icon_paths_;
+  std::unordered_map<std::string, EntryLocationMap> cached_locations_;
 };
 
 
 // Public Methods
 DBusIndicators::Impl::Impl(std::string const& dbus_name, DBusIndicators* owner)
   : owner_(owner)
-  , gproxy_(dbus_name, SERVICE_PATH, SERVICE_IFACE,
-            G_BUS_TYPE_SESSION, G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES)
+  , gproxy_(dbus_name, UPS_PATH, UPS_IFACE, G_BUS_TYPE_SESSION,
+            G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES)
 {
   gproxy_.Connect("ReSync", sigc::mem_fun(this, &DBusIndicators::Impl::OnReSync));
+  gproxy_.Connect("IconPathsChanged", sigc::mem_fun(this, &DBusIndicators::Impl::OnIconsPathChanged));
   gproxy_.Connect("EntryActivated", sigc::mem_fun(this, &DBusIndicators::Impl::OnEntryActivated));
   gproxy_.Connect("EntryActivateRequest", sigc::mem_fun(this, &DBusIndicators::Impl::OnEntryActivatedRequest));
   gproxy_.Connect("EntryShowNowChanged", sigc::mem_fun(this, &DBusIndicators::Impl::OnEntryShowNowChanged));
@@ -127,6 +141,7 @@ void DBusIndicators::Impl::CheckLocalService()
 
 void DBusIndicators::Impl::OnConnected()
 {
+  OnIconsPathChanged(nullptr);
   RequestSyncAll();
 }
 
@@ -157,18 +172,69 @@ void DBusIndicators::Impl::OnReSync(GVariant* parameters)
   }
 }
 
+void DBusIndicators::Impl::OnIconsPathChanged(GVariant*)
+{
+  gproxy_.CallBegin("GetIconPaths", nullptr, [this] (GVariant* paths, glib::Error const& e) {
+    if (e || !paths)
+    {
+      LOG_ERROR(logger) << "Something went wrong on GetIconPaths: " << e;
+      return;
+    }
+
+    bool changed = false;
+    gsize length;
+    glib::Variant array(g_variant_get_child_value(paths, 0));
+    const gchar** icon_paths = g_variant_get_strv(array, &length);
+
+    if (icon_paths_.size() != length)
+    {
+      changed = true;
+    }
+    else
+    {
+      for (unsigned i = 0; i < length; ++i)
+      {
+        if (icon_paths_[i] != glib::gchar_to_string(icon_paths[i]))
+        {
+          changed = true;
+          break;
+        }
+      }
+    }
+
+    if (changed)
+    {
+      icon_paths_.resize(length);
+
+      for (unsigned i = 0; i < length; ++i)
+        icon_paths_[i] = glib::gchar_to_string(icon_paths[i]);
+
+      owner_->icon_paths_changed.emit();
+    }
+
+    g_free(icon_paths);
+  });
+}
+
 void DBusIndicators::Impl::OnEntryActivated(GVariant* parameters)
 {
-  glib::String entry_name;
-  nux::Rect geo;
-  g_variant_get(parameters, "(s(iiuu))", &entry_name, &geo.x, &geo.y, &geo.width, &geo.height);
+  if (!verify_variant_type(parameters, "(ss(iiuu))"))
+    return;
 
-  if (entry_name)
-    owner_->ActivateEntry(entry_name, geo);
+  glib::String panel;
+  glib::String entry_id;
+  nux::Rect geo;
+  g_variant_get(parameters, "(ss(iiuu))", &panel, &entry_id, &geo.x, &geo.y, &geo.width, &geo.height);
+
+  if (entry_id)
+    owner_->ActivateEntry(panel.Str(), entry_id.Str(), geo);
 }
 
 void DBusIndicators::Impl::OnEntryActivatedRequest(GVariant* parameters)
 {
+  if (!verify_variant_type(parameters, "(s)"))
+    return;
+
   glib::String entry_name;
   g_variant_get(parameters, "(s)", &entry_name);
 
@@ -178,6 +244,9 @@ void DBusIndicators::Impl::OnEntryActivatedRequest(GVariant* parameters)
 
 void DBusIndicators::Impl::OnEntryShowNowChanged(GVariant* parameters)
 {
+  if (!verify_variant_type(parameters, "(sb)"))
+    return;
+
   glib::String entry_name;
   gboolean show_now;
   g_variant_get(parameters, "(sb)", &entry_name, &show_now);
@@ -188,14 +257,14 @@ void DBusIndicators::Impl::OnEntryShowNowChanged(GVariant* parameters)
 
 void DBusIndicators::Impl::RequestSyncAll()
 {
-  gproxy_.Call("Sync", nullptr, sigc::mem_fun(this, &DBusIndicators::Impl::Sync));
+  gproxy_.CallBegin("Sync", nullptr, sigc::mem_fun(this, &DBusIndicators::Impl::Sync));
 }
 
 void DBusIndicators::Impl::RequestSyncIndicator(std::string const& name)
 {
   GVariant* parameter = g_variant_new("(s)", name.c_str());
 
-  gproxy_.Call("SyncOne", parameter, sigc::mem_fun(this, &DBusIndicators::Impl::Sync));
+  gproxy_.CallBegin("SyncOne", parameter, sigc::mem_fun(this, &DBusIndicators::Impl::Sync));
 }
 
 
@@ -210,24 +279,46 @@ void DBusIndicators::Impl::OnEntryShowMenu(std::string const& entry_id,
   // menu not to show
 
   show_entry_idle_.reset(new glib::Idle(glib::Source::Priority::DEFAULT));
-  show_entry_idle_->Run([&, entry_id, xid, x, y, button] {
+  show_entry_idle_->Run([this, entry_id, xid, x, y, button] {
     gproxy_.Call("ShowEntry", g_variant_new("(suiiu)", entry_id.c_str(), xid,
                                                        x, y, button));
     return false;
   });
 }
 
+void DBusIndicators::Impl::ShowEntriesDropdown(Indicator::Entries const& entries, Entry::Ptr const& selected, unsigned xid, int x, int y)
+{
+  if (entries.empty())
+    return;
+
+  auto const& selected_id = (selected) ? selected->id() : "";
+  owner_->on_entry_show_menu.emit(selected ? selected_id : "dropdown", xid, x, y, 0);
+
+  GVariantBuilder builder;
+  g_variant_builder_init(&builder, G_VARIANT_TYPE_ARRAY);
+  for (auto const& entry : entries)
+    g_variant_builder_add(&builder, "s", entry->id().c_str());
+
+  glib::Variant parameters(g_variant_new("(assuii)", &builder, selected_id.c_str(), xid, x, y));
+
+  show_entry_idle_.reset(new glib::Idle(glib::Source::Priority::DEFAULT));
+  show_entry_idle_->Run([this, parameters] {
+    gproxy_.Call("ShowEntriesDropdown", parameters);
+    return false;
+  });
+}
+
 void DBusIndicators::Impl::OnShowAppMenu(unsigned int xid, int x, int y)
 {
-  owner_->on_show_appmenu.emit(xid, x, y);
+  owner_->on_entry_show_menu.emit("appmenu", xid, x, y, 0);
 
   // We have to do this because on certain systems X won't have time to
   // respond to our request for XUngrabPointer and this will cause the
   // menu not to show
 
   show_entry_idle_.reset(new glib::Idle(glib::Source::Priority::DEFAULT));
-  show_entry_idle_->Run([&, xid, x, y] {
-    gproxy_.Call("ShowEntry", g_variant_new("(uii)", xid, x, y));
+  show_entry_idle_->Run([this, xid, x, y] {
+    gproxy_.Call("ShowAppMenu", g_variant_new("(uii)", xid, x, y));
     return false;
   });
 }
@@ -242,15 +333,21 @@ void DBusIndicators::Impl::OnEntryScroll(std::string const& entry_id, int delta)
   gproxy_.Call("ScrollEntry", g_variant_new("(si)", entry_id.c_str(), delta));
 }
 
-void DBusIndicators::Impl::Sync(GVariant* args)
+void DBusIndicators::Impl::CloseActiveEntry()
 {
-  if (!args)
+  gproxy_.Call("CloseActiveEntry");
+}
+
+void DBusIndicators::Impl::Sync(GVariant* args, glib::Error const& error)
+{
+  if (!args || error)
     return;
 
   GVariantIter* iter            = nullptr;
-  gchar*        name_hint       = nullptr;
   gchar*        indicator_id    = nullptr;
   gchar*        entry_id        = nullptr;
+  gchar*        name_hint       = nullptr;
+  guint32       parent_window   = 0;
   gchar*        label           = nullptr;
   gboolean      label_sensitive = false;
   gboolean      label_visible   = false;
@@ -260,15 +357,17 @@ void DBusIndicators::Impl::Sync(GVariant* args)
   gboolean      image_visible   = false;
   gint32        priority        = -1;
 
-  std::map<Indicator::Ptr, Indicator::Entries> indicators;
-  int wanted_idx = 0;
-  bool any_different_idx = false;
+  if (!verify_variant_type(args, "(" ENTRY_ARRAY_SIGNATURE ")"))
+    return;
 
-  g_variant_get(args, "(a(ssssbbusbbi))", &iter);
-  while (g_variant_iter_loop(iter, "(ssssbbusbbi)",
+  std::unordered_map<Indicator::Ptr, Indicator::Entries> indicators;
+
+  g_variant_get(args, "(" ENTRY_ARRAY_SIGNATURE ")", &iter);
+  while (g_variant_iter_loop(iter, ENTRY_SIGNATURE,
                              &indicator_id,
                              &entry_id,
                              &name_hint,
+                             &parent_window,
                              &label,
                              &label_sensitive,
                              &label_visible,
@@ -292,48 +391,35 @@ void DBusIndicators::Impl::Sync(GVariant* args)
     // Empty entries are empty indicators.
     if (!entry.empty())
     {
-      Entry::Ptr e;
-      if (!any_different_idx)
-      {
-        // Indicators can only add or remove entries, so if
-        // there is a index change we can't reuse the existing ones
-        // after that index
-        if (indicator->EntryIndex(entry_id) == wanted_idx)
-        {
-          e = indicator->GetEntry(entry_id);
-        }
-        else
-        {
-          any_different_idx = true;
-        }
-      }
+      Entry::Ptr e = indicator->GetEntry(entry_id);
+      int old_priority = e ? e->priority() : -1;
+
+      // Indicators can only add or remove entries, so if
+      // there is a priority change we can't reuse the existing ones
+      if (old_priority != priority)
+        e.reset();
 
       if (!e)
       {
-        e = std::make_shared<Entry>(entry, name_hint, label, label_sensitive,
-                                    label_visible, image_type, image_data,
-                                    image_sensitive, image_visible, priority);
+        e = std::make_shared<Entry>(entry, name_hint, parent_window,
+                                    label, label_sensitive, label_visible,
+                                    image_type, image_data, image_sensitive, image_visible,
+                                    priority);
       }
       else
       {
-        e->setLabel(label, label_sensitive, label_visible);
-        e->setImage(image_type, image_data, image_sensitive, image_visible);
-        e->setPriority(priority);
+        e->set_label(label, label_sensitive, label_visible);
+        e->set_image(image_type, image_data, image_sensitive, image_visible);
+        e->set_priority(priority);
       }
 
       entries.push_back(e);
-      ++wanted_idx;
     }
   }
   g_variant_iter_free(iter);
 
-  for (auto i = indicators.begin(), end = indicators.end(); i != end; ++i)
-  {
-    i->first->Sync(indicators[i->first]);
-  }
-
-  // Notify listeners we have new data
-  owner_->on_synced.emit();
+  for (auto const& i : indicators)
+    i.first->Sync(i.second);
 }
 
 void DBusIndicators::Impl::SyncGeometries(std::string const& name,
@@ -384,18 +470,22 @@ void DBusIndicators::Impl::SyncGeometries(std::string const& name,
 
   g_variant_builder_close(&b);
 
-  gproxy_.Call("SyncGeometries", g_variant_builder_end(&b));
+  gproxy_.CallBegin("SyncGeometries", g_variant_builder_end(&b), [] (GVariant*, glib::Error const& e) {});
   cached_locations = locations;
 }
 
 DBusIndicators::DBusIndicators()
-  : pimpl(new Impl(SERVICE_NAME, this))
+  : pimpl(new Impl(UPS_NAME_DESKTOP, this))
 {}
 
 DBusIndicators::DBusIndicators(std::string const& dbus_name)
   : pimpl(new Impl(dbus_name, this))
 {}
 
+LockScreenDBusIndicators::LockScreenDBusIndicators()
+  : DBusIndicators(UPS_NAME_LOCKSCREEN)
+{}
+
 DBusIndicators::~DBusIndicators()
 {}
 
@@ -410,6 +500,18 @@ void DBusIndicators::SyncGeometries(std::string const& name,
   pimpl->SyncGeometries(name, locations);
 }
 
+void DBusIndicators::ShowEntriesDropdown(Indicator::Entries const& entries,
+                                         Entry::Ptr const& selected,
+                                         unsigned xid, int x, int y)
+{
+  pimpl->ShowEntriesDropdown(entries, selected, xid, x, y);
+}
+
+void DBusIndicators::CloseActiveEntry()
+{
+  pimpl->CloseActiveEntry();
+}
+
 void DBusIndicators::OnEntryScroll(std::string const& entry_id, int delta)
 {
   pimpl->OnEntryScroll(entry_id, delta);
@@ -432,5 +534,10 @@ void DBusIndicators::OnShowAppMenu(unsigned int xid, int x, int y)
   pimpl->OnShowAppMenu(xid, x, y);
 }
 
+std::vector<std::string> const& DBusIndicators::IconPaths() const
+{
+  return pimpl->icon_paths_;
+}
+
 } // namespace indicator
 } // namespace unity
