@@ -59,7 +59,7 @@ const RawPixel PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET = 10_em;
 const RawPixel PREVIEW_CONTAINER_TRIANGLE_WIDTH = 14_em;
 const RawPixel PREVIEW_CONTAINER_TRIANGLE_HEIGHT = 12_em;
 
-const int MAX_ENTRY_ACTIVATE_WAIT_TIMEOUT = 1000;
+const int MAX_ENTRY_ACTIVATE_WAIT_TIMEOUT = 300;
 }
 
 // This is so we can access some protected members in nux::VLayout and
@@ -120,7 +120,6 @@ DashView::DashView(Scopes::Ptr const& scopes, ApplicationStarter::Ptr const& app
   , preview_displaying_(false)
   , preview_navigation_mode_(previews::Navigation::NONE)
   , last_activated_timestamp_(0)
-  , search_in_progress_(false)
   , activate_on_finish_(false)
   , visible_(false)
   , opening_column_x_(-1)
@@ -482,6 +481,7 @@ void DashView::AboutToShow(int monitor)
 
     // this will make sure the spinner animates if the search takes a while
     search_bar_->ForceLiveSearch();
+    search_bar_->search_hint = active_scope_view_->scope()->search_hint;
   }
 
   // if a preview is open, close it
@@ -1174,7 +1174,9 @@ std::string DashView::AnalyseScopeURI(std::string const& uri)
 
   if (!filter.filters.empty())
   {
-    scope_views_[filter.id]->filters_expanded = true;
+    if (scope_views_.find(filter.id) != std::end(scope_views_))
+      scope_views_[filter.id]->filters_expanded = true;
+
     // update the scope for each filter
     for (auto p : filter.filters) {
       UpdateScopeFilter(filter.id, p.first, p.second);
@@ -1219,21 +1221,27 @@ void DashView::UpdateScopeFilterValue(Filter::Ptr filter, std::string value)
 
 void DashView::OnSearchChanged(std::string const& search_string)
 {
-  search_in_progress_ = true;
+  activate_on_finish_ = false;
 }
 
 void DashView::OnLiveSearchReached(std::string const& search_string)
 {
-  // reset and set it again once we're sure a search is happening
-  search_in_progress_ = false;
-
   LOG_DEBUG(logger) << "Live search reached: " << search_string;
-  if (active_scope_view_)
+  if (!active_scope_view_.IsValid())
+    return;
+
+  if (active_scope_view_->PerformSearch(search_string, sigc::mem_fun(this, &DashView::OnScopeSearchFinished)))
   {
-    if (active_scope_view_->PerformSearch(search_string, sigc::mem_fun(this, &DashView::OnScopeSearchFinished)))
-    {
-      search_in_progress_ = true;
-    }
+    activate_delay_.reset(new glib::Timeout(MAX_ENTRY_ACTIVATE_WAIT_TIMEOUT, [this] {
+      if (activate_on_finish_)
+      {
+        activate_on_finish_ = false;
+        active_scope_view_->ActivateFirst();
+      }
+
+      activate_delay_.reset();
+      return false;
+    }));
   }
 }
 
@@ -1252,12 +1260,15 @@ void DashView::OnScopeSearchFinished(std::string const& scope_id, std::string co
       LOG_DEBUG(logger) << "Search completed: " << search_string;
 
     search_bar_->SetSearchFinished();
-    search_in_progress_ = false;
 
-    activate_timeout_.reset();
-    if (activate_on_finish_ && !err)
-      OnEntryActivated();
-    activate_on_finish_= false;
+    if (activate_on_finish_)
+    {
+      activate_on_finish_ = false;
+      activate_delay_.reset();
+
+      if (!err)
+        active_scope_view_->ActivateFirst();
+    }
   }
 }
 
@@ -1405,20 +1416,12 @@ void DashView::DisableBlur()
 }
 void DashView::OnEntryActivated()
 {
-  if (active_scope_view_.IsValid() && !search_in_progress_)
+  if (active_scope_view_.IsValid())
   {
-    active_scope_view_->ActivateFirst();
-  }
-  // delay the activation until we get the SearchFinished signal
-  activate_on_finish_ = search_in_progress_;
-
-  if (activate_on_finish_)
-  {
-    activate_timeout_.reset(new glib::Timeout(MAX_ENTRY_ACTIVATE_WAIT_TIMEOUT, [this] {
-      activate_on_finish_ = false;
+    if (!activate_delay_ && !search_bar_->in_live_search())
       active_scope_view_->ActivateFirst();
-      return FALSE;
-    }));
+    else
+      activate_on_finish_ = true;
   }
 }
 
@@ -1544,6 +1547,32 @@ void DashView::ProcessDndEnter()
     ubus_manager_.SendMessage(UBUS_OVERLAY_CLOSE_REQUEST);
 }
 
+nux::Area* DashView::SkipUnexpandableHeaderKeyNav()
+{
+  PlacesGroup::Ptr prev_view;
+  auto category_views = active_scope_view_->GetOrderedCategoryViews();
+
+  for (auto category : category_views)
+  {
+    if (category->GetLayout() != nullptr)
+    {
+      auto header = category->GetHeaderFocusableView();
+      if (header && header->HasKeyFocus() && !category->IsExpandable())
+      {
+        if (prev_view)
+          return prev_view->GetChildView();
+        else
+          return search_bar_->text_entry();
+      }
+
+      if (category->IsVisible())
+        prev_view = category;
+    }
+  }
+
+  return nullptr;
+}
+
 nux::Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
                                       unsigned long x11_key_code,
                                       unsigned long special_keys_state)
@@ -1574,11 +1603,9 @@ nux::Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
     direction = KEY_NAV_RIGHT;
     break;
   case NUX_VK_LEFT_TAB:
-  case NUX_VK_PAGE_UP:
     direction = KEY_NAV_TAB_PREVIOUS;
     break;
   case NUX_VK_TAB:
-  case NUX_VK_PAGE_DOWN:
     direction = KEY_NAV_TAB_NEXT;
     break;
   case NUX_VK_ENTER:
@@ -1586,6 +1613,14 @@ nux::Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
     // Not sure if Enter should be a navigation key
     direction = KEY_NAV_ENTER;
     break;
+  case NUX_VK_PAGE_UP:
+  case NUX_VK_PAGE_DOWN:
+    if (!preview_displaying_)
+    {
+      active_scope_view_->PerformPageNavigation((x11_key_code == NUX_VK_PAGE_UP) ? ScrollDir::UP : ScrollDir::DOWN);
+      return nux::GetWindowCompositor().GetKeyFocusArea();
+    }
+    break;
   default:
     auto const& close_key = WindowManager::Default().close_window_key();
 
@@ -1689,6 +1724,14 @@ nux::Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
     }
   }
 
+  if (direction == KEY_NAV_UP)
+  {
+    if (auto skip_view = SkipUnexpandableHeaderKeyNav())
+    {
+      return skip_view;
+    }
+  }
+
   bool search_key = false;
 
   if (direction == KEY_NAV_NONE)
