@@ -33,7 +33,6 @@
 #include "unity-shared/UBusWrapper.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/GraphicsUtils.h"
-#include "unity-shared/RawPixel.h"
 
 #include "ResultView.h"
 #include "ResultViewGrid.h"
@@ -133,10 +132,8 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
   SetAcceptKeyNavFocusOnMouseEnter(false);
   scale.changed.connect(sigc::mem_fun(this, &PlacesGroup::UpdateScale));
 
-  nux::BaseTexture* arrow = _style.GetGroupExpandIcon();
-
-  _background = _style.GetCategoryBackground();
-  _background_nofilters = _style.GetCategoryBackgroundNoFilters();
+  _background = _style.GetCategoryBackground().GetPointer();
+  _background_nofilters = _style.GetCategoryBackgroundNoFilters().GetPointer();
 
   nux::ROPConfig rop;
   rop.Blend = true;
@@ -144,8 +141,8 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
   rop.DstBlend = GL_ONE_MINUS_SRC_ALPHA;
 
   nux::TexCoordXForm texxform;
-  _background_layer.reset(new nux::TextureLayer(_background_nofilters->GetDeviceTexture(), 
-                          texxform, 
+  _background_layer.reset(new nux::TextureLayer(_background_nofilters->GetDeviceTexture(),
+                          texxform,
                           nux::color::White,
                           false,
                           rop));
@@ -160,11 +157,10 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
   _group_layout->AddView(_header_view, 0, nux::MINOR_POSITION_START, nux::MINOR_SIZE_FULL);
 
   _header_layout = new nux::HLayout(NUX_TRACKER_LOCATION);
-  _header_layout->SetLeftAndRightPadding(_style.GetCategoryHeaderLeftPadding(), 0);
+  _header_layout->SetLeftAndRightPadding(_style.GetCategoryHeaderLeftPadding().CP(scale), 0);
   _header_view->SetLayout(_header_layout);
 
-  RawPixel const icon_size = _style.GetCategoryIconSize();
-  _icon = new IconTexture("", icon_size.CP(scale()));
+  _icon = new IconTexture("", _style.GetCategoryIconSize().CP(scale));
   _header_layout->AddView(_icon, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FIX);
 
   _text_layout = new nux::HLayout(NUX_TRACKER_LOCATION);
@@ -172,6 +168,7 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
 
   _name = new StaticCairoText("", NUX_TRACKER_LOCATION);
   _name->SetFont(NAME_LABEL_FONT);
+  _name->SetLines(-1);
   _name->SetTextEllipsize(StaticCairoText::NUX_ELLIPSIZE_END);
   _name->SetTextAlignment(StaticCairoText::NUX_ALIGN_LEFT);
   _text_layout->AddView(_name, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
@@ -184,16 +181,17 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
 
   _expand_label = new StaticCairoText("", NUX_TRACKER_LOCATION);
   _expand_label->SetFont(EXPANDER_LABEL_FONT);
+  _expand_label->SetLines(-1);
   _expand_label->SetTextEllipsize(StaticCairoText::NUX_ELLIPSIZE_END);
   _expand_label->SetTextAlignment(StaticCairoText::NUX_ALIGN_LEFT);
   _expand_label->SetTextColor(kExpandDefaultTextColor);
   _expand_label_layout->AddView(_expand_label, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FIX);
 
-  _expand_icon = new IconTexture(arrow, arrow->GetWidth(), arrow->GetHeight());
+  _expand_icon = new IconTexture(_style.GetGroupExpandIcon());
+  _expand_icon->SetDrawMode(IconTexture::DrawMode::STRETCH_WITH_ASPECT);
   _expand_icon->SetOpacity(kExpandDefaultIconOpacity);
-  _expand_icon->SetMinimumSize(arrow->GetWidth(), arrow->GetHeight());
   _expand_icon->SetVisible(false);
-  _expand_layout->AddView(_expand_icon, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FIX);
+  _expand_layout->AddView(_expand_icon, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
 
   SetLayout(_group_layout);
 
@@ -223,16 +221,16 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
 void
 PlacesGroup::UpdatePlacesGroupSize()
 {
-   RawPixel const icon_size = _style.GetCategoryIconSize();
-   RawPixel const group_top = _style.GetPlacesGroupTopSpace();
+   int icon_size = _style.GetCategoryIconSize().CP(scale);
+   int top_space = _style.GetPlacesGroupTopSpace().CP(scale);
 
-   int top_space = group_top.CP(scale());
    _space_layout->SetMinimumSize(top_space, top_space);
    _space_layout->SetMaximumSize(top_space, top_space);
 
   _header_layout->SetSpaceBetweenChildren(SPACE_BETWEEN_CHILDREN.CP(scale()));
+  _header_layout->SetLeftAndRightPadding(_style.GetCategoryHeaderLeftPadding().CP(scale), 0);
 
-  _icon->SetMinMaxSize(icon_size.CP(scale()), icon_size.CP(scale()));
+  _icon->SetMinMaxSize(icon_size, icon_size);
 
   _text_layout->SetHorizontalInternalMargin(TEXT_INTERNAL_MARGIN.CP(scale()));
   _expand_layout->SetHorizontalInternalMargin(EXPAND_INTERNAL_MARGIN.CP(scale()));
@@ -241,15 +239,16 @@ PlacesGroup::UpdatePlacesGroupSize()
 void
 PlacesGroup::UpdateScale(double scale)
 {
-  RawPixel const icon_size = _style.GetCategoryIconSize();
-
+  _name->SetMinimumSize(nux::AREA_MIN_WIDTH, nux::AREA_MIN_HEIGHT);
+  _name->SetMaximumSize(nux::AREA_MAX_WIDTH, nux::AREA_MAX_HEIGHT);
   _name->SetScale(scale);
   _expand_label->SetScale(scale);
 
-  _icon->SetSize(icon_size.CP(scale));
+  _icon->SetSize(_style.GetCategoryIconSize().CP(scale));
   _icon->ReLoadIcon();
 
-  // FIXME _expand_icon, needs some work here. Not as easy as _icon
+  auto const& arrow = _expand_icon->texture();
+  _expand_icon->SetMinMaxSize(RawPixel(arrow->GetWidth()).CP(scale), RawPixel(arrow->GetHeight()).CP(scale));
 
   if (_child_view)
     _child_view->scale = scale;
@@ -279,7 +278,6 @@ PlacesGroup::OnLabelFocusChanged(nux::Area* label, bool has_focus, nux::KeyNavDi
 
 void
 
-// FIXME _expand_icon, needs some work here. Not as easy as _icon
 PlacesGroup::SetName(std::string const& name)
 {
   if (_cached_name != name)
@@ -310,7 +308,7 @@ PlacesGroup::GetExpandLabel()
 void
 PlacesGroup::SetIcon(std::string const& path_to_emblem)
 {
-  _icon->SetByIconName(path_to_emblem, _style.GetCategoryIconSize());
+  _icon->SetByIconName(path_to_emblem, _style.GetCategoryIconSize().CP(scale));
 }
 
 void
@@ -318,11 +316,8 @@ PlacesGroup::UpdateResultViewPadding()
 {
   if (_child_layout)
   {
-    RawPixel const result_top_padding  = _style.GetPlacesGroupResultTopPadding();
-    RawPixel const result_left_padding = _style.GetPlacesGroupResultLeftPadding();
-
-    _child_layout->SetTopAndBottomPadding(result_top_padding.CP(scale()), 0);
-    _child_layout->SetLeftAndRightPadding(result_left_padding.CP(scale()), 0);
+    _child_layout->SetTopAndBottomPadding(_style.GetPlacesGroupResultTopPadding().CP(scale), 0);
+    _child_layout->SetLeftAndRightPadding(_style.GetPlacesGroupResultLeftPadding().CP(scale), 0);
   }
 }
 
@@ -462,7 +457,7 @@ long PlacesGroup::ComputeContentSize()
   // only the width matters
   if (_cached_geometry.GetWidth() != geo.GetWidth())
   {
-    _focus_layer.reset(_style.FocusOverlay(geo.width - 
+    _focus_layer.reset(_style.FocusOverlay(geo.width -
                                            kHighlightLeftPadding.CP(scale()) -
                                            kHighlightRightPadding.CP(scale()),
                                            kHighlightHeight.CP(scale())));
@@ -595,6 +590,9 @@ PlacesGroup::SetExpanded(bool is_expanded)
   else
     _expand_icon->SetTexture(_style.GetGroupExpandIcon());
 
+  auto const& tex = _expand_icon->texture();
+  _expand_icon->SetMinMaxSize(RawPixel(tex->GetWidth()).CP(scale), RawPixel(tex->GetHeight()).CP(scale));
+
   expanded.emit(this);
 }
 
