@@ -21,13 +21,12 @@
  */
 
 #include "PreviewContainer.h"
-#include <NuxCore/Logger.h>
 #include <Nux/HLayout.h>
-#include <Nux/VLayout.h>
 
 #include "unity-shared/IntrospectableWrappers.h"
 #include "unity-shared/TimeUtil.h"
 #include "unity-shared/PreviewStyle.h"
+#include "unity-shared/DashStyle.h"
 #include "unity-shared/GraphicsUtils.h"
 #include "PreviewNavigator.h"
 #include <boost/math/constants/constants.hpp>
@@ -39,7 +38,6 @@ namespace dash
 {
 namespace previews
 {
-DECLARE_LOGGER(logger, "unity.dash.preview.container");
 
 Navigation operator&(const Navigation lhs, const Navigation rhs)
 {
@@ -52,13 +50,15 @@ const int ANIM_DURATION_LONG = 500;
 const int PREVIEW_SPINNER_WAIT = 2000;
 
 const std::string ANIMATION_IDLE = "animation-idle";
+const RawPixel CHILDREN_SPACE = 6_em;
 }
 
 class PreviewContent : public nux::Layout, public debug::Introspectable
 {
 public:
   PreviewContent(PreviewContainer*const parent)
-  : parent_(parent)
+  : scale(1.0)
+  , parent_(parent)
   , progress_(0.0)
   , curve_progress_(0.0)
   , animating_(false)
@@ -73,9 +73,19 @@ public:
       // Need to update the preview geometries when updating the container geo.
       UpdateAnimationProgress(progress_, curve_progress_);
     });
-    Style& style = previews::Style::Instance();
 
-    spin_= style.GetSearchSpinIcon(32);
+    spin_ = dash::Style::Instance().GetSearchSpinIcon(scale);
+    scale.changed.connect(sigc::mem_fun(this, &PreviewContent::UpdateScale));
+  }
+
+  void UpdateScale(double scale)
+  {
+    spin_ = dash::Style::Instance().GetSearchSpinIcon(scale);
+
+    for (auto* area : GetChildren())
+      static_cast<previews::Preview*>(area)->scale = scale;
+
+    QueueDraw();
   }
 
   // From debug::Introspectable
@@ -107,6 +117,7 @@ public:
       AddChild(preview.GetPointer());
       AddView(preview.GetPointer());
       preview->SetVisible(false);
+      preview->scale = scale();
     }
     else
     {
@@ -355,6 +366,7 @@ public:
   sigc::signal<void> start_navigation;
   sigc::signal<void> continue_navigation;
   sigc::signal<void> end_navigation;
+  nux::Property<double> scale;
 
 private:
   PreviewContainer*const parent_;
@@ -394,6 +406,7 @@ NUX_IMPLEMENT_OBJECT_TYPE(PreviewContainer);
 
 PreviewContainer::PreviewContainer(NUX_FILE_LINE_DECL)
   : View(NUX_FILE_LINE_PARAM)
+  , scale(1.0)
   , preview_layout_(nullptr)
   , nav_disabled_(Navigation::NONE)
   , navigation_progress_speed_(0.0)
@@ -408,6 +421,7 @@ PreviewContainer::PreviewContainer(NUX_FILE_LINE_DECL)
 
   key_down.connect(sigc::mem_fun(this, &PreviewContainer::OnKeyDown));
   mouse_click.connect(sigc::mem_fun(this, &PreviewContainer::OnMouseDown));
+  scale.changed.connect(sigc::mem_fun(this, &PreviewContainer::UpdateScale));
 }
 
 PreviewContainer::~PreviewContainer()
@@ -417,13 +431,12 @@ PreviewContainer::~PreviewContainer()
 void PreviewContainer::Preview(dash::Preview::Ptr preview_model, Navigation direction)
 {
   previews::Preview::Ptr preview_view = preview_model ? previews::Preview::PreviewForModel(preview_model) : previews::Preview::Ptr();
-  
+
   if (preview_view)
   {
     preview_view->request_close().connect([this]() { request_close.emit(); });
+    preview_layout_->PushPreview(preview_view, direction);
   }
-  
-  preview_layout_->PushPreview(preview_view, direction);
 }
 
 void PreviewContainer::DisableNavButton(Navigation button)
@@ -458,29 +471,32 @@ void PreviewContainer::SetupViews()
 
   nux::VLayout* layout = new nux::VLayout();
   SetLayout(layout);
-  layout->AddLayout(new nux::SpaceLayout(0,0,style.GetPreviewTopPadding(),style.GetPreviewTopPadding()));
+
+  layout->SetTopAndBottomPadding(style.GetPreviewTopPadding().CP(scale), 0);
 
   layout_content_ = new nux::HLayout();
-  layout_content_->SetSpaceBetweenChildren(6);
+  layout_content_->SetSpaceBetweenChildren(CHILDREN_SPACE.CP(scale));
   layout->AddLayout(layout_content_, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
 
   layout_content_->AddSpace(0, 1);
   nav_left_ = new PreviewNavigator(Orientation::LEFT, NUX_TRACKER_LOCATION);
   AddChild(nav_left_);
-  nav_left_->SetMinimumWidth(style.GetNavigatorWidth());
-  nav_left_->SetMaximumWidth(style.GetNavigatorWidth());
+  nav_left_->scale = scale();
+  nav_left_->SetMinimumWidth(style.GetNavigatorWidth().CP(scale));
+  nav_left_->SetMaximumWidth(style.GetNavigatorWidth().CP(scale));
   nav_left_->activated.connect([this]() { navigate_left.emit(); });
   layout_content_->AddView(nav_left_, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
 
   preview_layout_ = new PreviewContent(this);
-  preview_layout_->SetMinMaxSize(style.GetPreviewWidth(), style.GetPreviewHeight());
+  preview_layout_->SetMinMaxSize(style.GetPreviewWidth().CP(scale), style.GetPreviewHeight().CP(scale));
   AddChild(preview_layout_);
   layout_content_->AddLayout(preview_layout_, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
 
   nav_right_ = new PreviewNavigator(Orientation::RIGHT, NUX_TRACKER_LOCATION);
   AddChild(nav_right_);
-  nav_right_->SetMinimumWidth(style.GetNavigatorWidth());
-  nav_right_->SetMaximumWidth(style.GetNavigatorWidth());
+  nav_right_->scale = scale();
+  nav_right_->SetMinimumWidth(style.GetNavigatorWidth().CP(scale));
+  nav_right_->SetMaximumWidth(style.GetNavigatorWidth().CP(scale));
   nav_right_->activated.connect([this]() { navigate_right.emit(); });
   layout_content_->AddView(nav_right_, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
   layout_content_->AddSpace(0, 1);
@@ -492,7 +508,7 @@ void PreviewContainer::SetupViews()
     // reset animation clock.
     if (navigation_count_ == 0)
       clock_gettime(CLOCK_MONOTONIC, &last_progress_time_);
-  
+
     float navigation_progress_remaining = CLAMP((1.0 - preview_layout_->GetAnimationProgress()) + navigation_count_, 1.0f, 10.0f);
     navigation_count_++;
 
@@ -534,7 +550,7 @@ void PreviewContainer::DrawContent(nux::GraphicsEngine& gfx_engine, bool force_d
     gfx_engine.GetRenderStates().SetBlend(false);
     gfx_engine.QRP_Color(GetX(), GetY(), GetWidth(), GetHeight(), nux::Color(0.0f, 0.0f, 0.0f, 0.0f));
   }
-  
+
     // rely on the compiz event loop to come back to us in a nice throttling
   if (AnimationInProgress())
   {
@@ -595,11 +611,12 @@ float PreviewContainer::GetSwipeAnimationProgress(struct timespec const& current
 bool PreviewContainer::QueueAnimation()
 {
   animation_timer_.reset();
-  
+
   timespec current;
   clock_gettime(CLOCK_MONOTONIC, &current);
   float progress = GetSwipeAnimationProgress(current);
-  preview_layout_->UpdateAnimationProgress(progress, easeInOutQuart(progress)); // ease in/out.
+  if (preview_layout_)
+    preview_layout_->UpdateAnimationProgress(progress, easeInOutQuart(progress)); // ease in/out.
   last_progress_time_ = current;
 
   QueueDraw();
@@ -691,7 +708,30 @@ void PreviewContainer::OnMouseDown(int x, int y, unsigned long button_flags, uns
 
 nux::Geometry PreviewContainer::GetLayoutGeometry() const
 {
-  return layout_content_->GetAbsoluteGeometry();  
+  return layout_content_->GetAbsoluteGeometry();
+}
+
+void PreviewContainer::UpdateScale(double scale)
+{
+  previews::Style& style = previews::Style::Instance();
+
+  GetLayout()->SetTopAndBottomPadding(style.GetPreviewTopPadding().CP(scale), 0);
+
+  preview_layout_->SetMinMaxSize(style.GetPreviewWidth().CP(scale), style.GetPreviewHeight().CP(scale));
+  preview_layout_->scale = scale;
+
+  layout_content_->SetSpaceBetweenChildren(CHILDREN_SPACE.CP(scale));
+
+  nav_left_->SetMinimumWidth(style.GetNavigatorWidth().CP(scale));
+  nav_left_->SetMaximumWidth(style.GetNavigatorWidth().CP(scale));
+  nav_left_->scale = scale;
+
+  nav_right_->SetMinimumWidth(style.GetNavigatorWidth().CP(scale));
+  nav_right_->SetMaximumWidth(style.GetNavigatorWidth().CP(scale));
+  nav_right_->scale = scale;
+
+  QueueRelayout();
+  QueueDraw();
 }
 
 } // namespace previews
