@@ -50,17 +50,13 @@ Window::Impl::Impl(Window* parent, CompWindow* win)
   , deco_elements_(cu::DecorationElement::NONE)
   , last_mwm_decor_(win_->mwmDecor())
   , last_actions_(win_->actions())
+  , panel_id_(MENUS_PANEL_NAME + std::to_string(win_->id()))
   , cv_(Settings::Instance().em())
 {
   active.changed.connect([this] (bool active) {
     bg_textures_.clear();
     if (top_layout_)
-    {
       top_layout_->focused = active;
-
-      if (!active)
-        UnsetAppMenu();
-    }
     RedrawDecorations();
   });
 
@@ -110,7 +106,12 @@ Window::Impl::~Impl()
 void Window::Impl::Update()
 {
   UpdateElements();
-  (deco_elements_ & (cu::DecorationElement::EDGE | cu::DecorationElement::BORDER)) ? Decorate() : Undecorate();
+
+  if (deco_elements_ & (cu::DecorationElement::EDGE | cu::DecorationElement::BORDER))
+    Decorate();
+  else
+    Undecorate();
+
   last_mwm_decor_ = win_->mwmDecor();
   last_actions_ = win_->actions();
 }
@@ -183,6 +184,13 @@ void Window::Impl::SetupExtents()
     win_->setWindowFrameExtents(&border, &input);
 }
 
+void Window::Impl::SendFrameExtents()
+{
+  UpdateElements(cu::WindowFilter::UNMAPPED);
+  SetupExtents();
+  win_->setWindowFrameExtents(&win_->border(), &win_->input());
+}
+
 void Window::Impl::UnsetFrame()
 {
   if (!frame_)
@@ -423,7 +431,7 @@ bool Window::Impl::IsMaximized() const
   return (win_->state() & MAXIMIZE_STATE) == MAXIMIZE_STATE;
 }
 
-void Window::Impl::UpdateElements()
+void Window::Impl::UpdateElements(cu::WindowFilter::Value wf)
 {
   if (!parent_->scaled() && IsMaximized())
   {
@@ -431,7 +439,7 @@ void Window::Impl::UpdateElements()
     return;
   }
 
-  deco_elements_ = cu::WindowDecorationElements(win_);
+  deco_elements_ = cu::WindowDecorationElements(win_, wf);
 }
 
 bool Window::Impl::ShadowDecorated() const
@@ -620,8 +628,10 @@ void Window::Impl::Paint(GLMatrix const& transformation,
                          GLWindowPaintAttrib const& attrib,
                          CompRegion const& region, unsigned mask)
 {
-  if (win_->defaultViewport() != screen->vp())
+  if (!(mask & PAINT_SCREEN_TRANSFORMED_MASK) && win_->defaultViewport() != screen->vp())
+  {
     return;
+  }
 
   if (dirty_geo_)
     parent_->UpdateDecorationPosition();
@@ -639,8 +649,10 @@ void Window::Impl::Draw(GLMatrix const& transformation,
                         GLWindowPaintAttrib const& attrib,
                         CompRegion const& region, unsigned mask)
 {
-  if (last_shadow_rect_.isEmpty() || win_->defaultViewport() != screen->vp())
+  if (last_shadow_rect_.isEmpty() || (!(mask & PAINT_SCREEN_TRANSFORMED_MASK) && win_->defaultViewport() != screen->vp()))
+  {
     return;
+  }
 
   auto const& clip_region = (mask & PAINT_WINDOW_TRANSFORMED_MASK) ? infiniteRegion : region;
   mask |= PAINT_WINDOW_BLEND_MASK;
@@ -685,39 +697,52 @@ void Window::Impl::RedrawDecorations()
 
 void Window::Impl::SetupAppMenu()
 {
-  if (!active() || !top_layout_)
+  if (!top_layout_)
     return;
 
   auto const& menu_manager = manager_->impl_->menu_manager_;
   auto const& sliding_layout = sliding_layout_.lock();
   sliding_layout->SetInputItem(nullptr);
   sliding_layout->mouse_owner = false;
+  sliding_layout->override_main_item = false;
+  grab_mouse_changed_->disconnect();
 
-  if (!menu_manager->HasAppMenu() || !Style::Get()->integrated_menus())
+  if (!menu_manager->HasAppMenu() || !menu_manager->integrated_menus())
     return;
 
-  auto visibility_cb = sigc::hide(sigc::mem_fun(this, &Impl::UpdateAppMenuVisibility));
   auto menus = std::make_shared<MenuLayout>(menu_manager, win_);
   menus->Setup();
-  menus->active.changed.connect(visibility_cb);
-  menus->show_now.changed.connect(visibility_cb);
-  menus->mouse_owner.changed.connect(visibility_cb);
-  menus_ = menus;
 
+  if (menus->Items().empty())
+    return;
+
+  menus_ = menus;
   auto const& grab_edge = grab_edge_.lock();
   sliding_layout->SetInputItem(menus);
   sliding_layout->fadein = menu_manager->fadein();
   sliding_layout->fadeout = menu_manager->fadeout();
 
-  if (grab_edge->mouse_owner() || grab_edge->Geometry().contains(CompPoint(pointerX, pointerY)))
-    sliding_layout->mouse_owner = true;
+  if (menu_manager->always_show_menus())
+  {
+    sliding_layout->override_main_item = true;
+  }
+  else
+  {
+    auto visibility_cb = sigc::hide(sigc::mem_fun(this, &Impl::UpdateAppMenuVisibility));
+    menus->active.changed.connect(visibility_cb);
+    menus->show_now.changed.connect(visibility_cb);
+    menus->mouse_owner.changed.connect(visibility_cb);
 
-  grab_mouse_changed_ = grab_edge->mouse_owner.changed.connect([this] (bool owner) {
-    sliding_layout_->mouse_owner = owner;
-  });
+    if (grab_edge->mouse_owner() || grab_edge->Geometry().contains(CompPoint(pointerX, pointerY)))
+      sliding_layout->mouse_owner = true;
 
-  if (sliding_layout->mouse_owner())
-    input_mixer_->ForceMouseOwnerCheck();
+    grab_mouse_changed_ = grab_edge->mouse_owner.changed.connect([this] (bool owner) {
+      sliding_layout_->mouse_owner = owner || menus_->show_now();
+    });
+
+    if (sliding_layout->mouse_owner())
+      input_mixer_->ForceMouseOwnerCheck();
+  }
 
   SyncMenusGeometries();
 }
@@ -733,13 +758,18 @@ void Window::Impl::UpdateAppMenuVisibility()
     sliding_layout->mouse_owner = grab_edge_->mouse_owner();
 }
 
+inline std::string const& Window::Impl::GetMenusPanelID() const
+{
+  return panel_id_;
+}
+
 void Window::Impl::UnsetAppMenu()
 {
   if (!menus_)
     return;
 
   auto const& indicators = manager_->impl_->menu_manager_->Indicators();
-  indicators->SyncGeometries(MENUS_PANEL_NAME, indicator::EntryLocationMap());
+  indicators->SyncGeometries(GetMenusPanelID(), indicator::EntryLocationMap());
   sliding_layout_->SetInputItem(nullptr);
   grab_mouse_changed_->disconnect();
 }
@@ -752,7 +782,7 @@ void Window::Impl::SyncMenusGeometries() const
   auto const& indicators = manager_->impl_->menu_manager_->Indicators();
   indicator::EntryLocationMap map;
   menus_->ChildrenGeometries(map);
-  indicators->SyncGeometries(MENUS_PANEL_NAME, map);
+  indicators->SyncGeometries(GetMenusPanelID(), map);
 }
 
 bool Window::Impl::ActivateMenu(std::string const& entry_id)
