@@ -58,7 +58,7 @@ Manager::Impl::Impl(decoration::Manager* parent, menu::Manager::Ptr const& menu)
   manager_->inactive_shadow_color.changed.connect(sigc::hide(sigc::bind(rebuild_cb, false)));
   manager_->inactive_shadow_radius.changed.connect(sigc::hide(sigc::bind(rebuild_cb, false)));
   manager_->shadow_offset.changed.connect(sigc::hide(sigc::mem_fun(this, &Impl::UpdateWindowsExtents)));
-  Style::Get()->integrated_menus.changed.connect(sigc::hide(sigc::mem_fun(this, &Impl::SetupIntegratedMenus)));
+  menu_manager_->integrated_menus.changed.connect(sigc::hide(sigc::mem_fun(this, &Impl::SetupIntegratedMenus)));
 
   BuildInactiveShadowTexture();
   BuildActiveShadowTexture();
@@ -110,7 +110,7 @@ void Manager::Impl::OnShadowOptionsChanged(bool active)
 
 void Manager::Impl::SetupIntegratedMenus()
 {
-  if (!Style::Get()->integrated_menus())
+  if (!menu_manager_->integrated_menus())
   {
     UnsetAppMenu();
     menu_connections_.Clear();
@@ -120,6 +120,7 @@ void Manager::Impl::SetupIntegratedMenus()
   menu_connections_.Add(menu_manager_->appmenu_added.connect(sigc::mem_fun(this, &Impl::SetupAppMenu)));
   menu_connections_.Add(menu_manager_->appmenu_removed.connect(sigc::mem_fun(this, &Impl::UnsetAppMenu)));
   menu_connections_.Add(menu_manager_->key_activate_entry.connect(sigc::mem_fun(this, &Impl::OnMenuKeyActivated)));
+  menu_connections_.Add(menu_manager_->always_show_menus.changed.connect(sigc::hide(sigc::mem_fun(this, &Impl::SetupAppMenu))));
 
   SetupAppMenu();
 }
@@ -140,25 +141,24 @@ void Manager::Impl::SetupAppMenu()
     return;
   }
 
-  auto setup_active_window = [this] {
-    if (Window::Ptr const& active_win = active_deco_win_.lock())
-      active_win->impl_->SetupAppMenu();
-  };
+  for (auto const& win : windows_)
+    win.second->impl_->SetupAppMenu();
 
   menu_connections_.Remove(appmenu_connection_);
-  appmenu_connection_ = menu_connections_.Add(appmenu->updated.connect(setup_active_window));
-  setup_active_window();
+  appmenu_connection_ = menu_connections_.Add(appmenu->updated_win.connect([this] (uint32_t xid) {
+    if (Window::Ptr const& win = GetWindowByXid(xid))
+      win->impl_->SetupAppMenu();
+  }));
 }
 
 void Manager::Impl::UnsetAppMenu()
 {
   menu_connections_.Remove(appmenu_connection_);
-  auto const& active_win = active_deco_win_.lock();
 
-  if (active_win)
+  for (auto const& win : windows_)
   {
-    active_win->impl_->UnsetAppMenu();
-    active_win->impl_->Damage();
+    win.second->impl_->UnsetAppMenu();
+    win.second->impl_->Damage();
   }
 }
 
@@ -209,7 +209,7 @@ bool Manager::Impl::HandleEventBefore(XEvent* event)
       if (event->xclient.message_type == atom::_NET_REQUEST_FRAME_EXTENTS)
       {
         if (Window::Ptr const& win = GetWindowByXid(event->xclient.window))
-          win->impl_->Decorate();
+          win->impl_->SendFrameExtents();
       }
       else if (event->xclient.message_type == Atoms::toolkitAction)
       {
@@ -256,7 +256,8 @@ bool Manager::Impl::HandleEventAfter(XEvent* event)
         if (active_deco_win_)
           active_deco_win_->impl_->active = false;
 
-        auto const& new_active = GetWindowByXid(screen->activeWindow());
+        auto active_xid = screen->activeWindow();
+        auto const& new_active = GetWindowByXid(active_xid);
         active_deco_win_ = new_active;
 
         if (new_active)
@@ -301,6 +302,9 @@ bool Manager::Impl::HandleEventAfter(XEvent* event)
 
 bool Manager::Impl::HandleFrameEvent(XEvent* event)
 {
+  if (WindowManager::Default().IsScaleActive())
+    return false;
+
   auto const& win = GetWindowByFrame(event->xany.window);
 
   // ButtonRelease events might happen also outside the frame window, in this
