@@ -33,6 +33,7 @@
 #include "MultiMonitor.h"
 #include "unity-shared/DesktopApplicationManager.h"
 #include "unity-shared/GnomeFileManager.h"
+#include "unity-shared/UBusWrapper.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/UScreen.h"
 
@@ -47,7 +48,6 @@ DECLARE_LOGGER(logger, "unity.launcher.icon.application");
 namespace
 {
 // We use the "bamf-" prefix since the manager is protected, to avoid name clash
-const std::string WINDOW_MOVE_TIMEOUT = "bamf-window-move";
 const std::string ICON_REMOVE_TIMEOUT = "bamf-icon-remove";
 const std::string ICON_DND_OVER_TIMEOUT = "bamf-icon-dnd-over";
 const std::string DEFAULT_ICON = "application-default-icon";
@@ -86,12 +86,11 @@ ApplicationLauncherIcon::ApplicationLauncherIcon(ApplicationPtr const& app)
 
   WindowManager& wm = WindowManager::Default();
   wm.window_minimized.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::OnWindowMinimized));
-  wm.window_moved.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::OnWindowMoved));
   wm.screen_viewport_switch_ended.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState));
   wm.terminate_expo.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState));
-  UScreen::GetDefault()->changed.connect(sigc::hide(sigc::hide(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState))));
+  UScreen::GetDefault()->changed.connect(sigc::hide(sigc::hide(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowsLocation))));
 
-  EnsureWindowState();
+  EnsureWindowsLocation();
 }
 
 ApplicationLauncherIcon::~ApplicationLauncherIcon()
@@ -155,13 +154,16 @@ void ApplicationLauncherIcon::SetupApplicationSignalsConnections()
 {
   // Lambda functions should be fine here because when the application the icon
   // is only ever removed when the application is closed.
-  signals_conn_.Add(app_->window_opened.connect([this](ApplicationWindow const&) {
-    EnsureWindowState();
-    UpdateIconGeometries(GetCenters());
+  signals_conn_.Add(app_->window_opened.connect([this](ApplicationWindowPtr const& win) {
+    signals_conn_.Add(win->monitor.changed.connect([this] (int) { EnsureWindowsLocation(); }));
+    EnsureWindowsLocation();
   }));
 
-  signals_conn_.Add(app_->window_closed.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState)));
-  signals_conn_.Add(app_->window_moved.connect(sigc::hide(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState))));
+  auto ensure_win_location_cb = sigc::hide(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowsLocation));
+  signals_conn_.Add(app_->window_closed.connect(ensure_win_location_cb));
+
+  for (auto& win : app_->GetWindows())
+    signals_conn_.Add(win->monitor.changed.connect(ensure_win_location_cb));
 
   signals_conn_.Add(app_->urgent.changed.connect([this](bool const& urgent) {
     LOG_DEBUG(logger) << tooltip_text() << " urgent now " << (urgent ? "true" : "false");
@@ -234,7 +236,7 @@ bool ApplicationLauncherIcon::GetQuirk(AbstractLauncherIcon::Quirk quirk, int mo
     if (!SimpleLauncherIcon::GetQuirk(Quirk::ACTIVE, monitor))
       return false;
 
-    if (app_->type() == "webapp")
+    if (app_->type() == AppType::WEBAPP)
       return true;
 
     // Sometimes BAMF is not fast enough to update the active application
@@ -533,19 +535,6 @@ void ApplicationLauncherIcon::OnWindowMinimized(guint32 xid)
   }
 }
 
-void ApplicationLauncherIcon::OnWindowMoved(guint32 moved_win)
-{
-  if (!app_->OwnsWindow(moved_win))
-    return;
-
-  _source_manager.AddTimeout(250, [this] {
-    EnsureWindowState();
-    UpdateIconGeometries(GetCenters());
-
-    return false;
-  }, WINDOW_MOVE_TIMEOUT);
-}
-
 void ApplicationLauncherIcon::UpdateDesktopFile()
 {
   std::string const& filename = app_->desktop_file();
@@ -570,14 +559,13 @@ void ApplicationLauncherIcon::UpdateDesktopFile()
     g_file_monitor_set_rate_limit(_desktop_file_monitor, 2000);
 
     _gsignals.Add<void, GFileMonitor*, GFile*, GFile*, GFileMonitorEvent>(_desktop_file_monitor, "changed",
-      [this] (GFileMonitor*, GFile* f, GFile*, GFileMonitorEvent event_type) {
+      [this, desktop_file] (GFileMonitor*, GFile*,  GFile*, GFileMonitorEvent event_type) {
       switch (event_type)
       {
         case G_FILE_MONITOR_EVENT_DELETED:
         {
-          glib::Object<GFile> file(f, glib::AddRef());
-          _source_manager.AddTimeoutSeconds(1, [this, file] {
-            if (!g_file_query_exists(file, nullptr))
+          _source_manager.AddTimeoutSeconds(1, [this, desktop_file] {
+            if (!g_file_query_exists(desktop_file, nullptr))
             {
               UnStick();
               LogUnityEvent(ApplicationEventType::DELETE);
@@ -701,7 +689,7 @@ void ApplicationLauncherIcon::Focus(ActionArg arg)
     if (window->Focus())
       return;
   }
-  else if (app_->type() == "webapp")
+  else if (app_->type() == AppType::WEBAPP)
   {
     // Webapps are again special.
     OpenInstanceLauncherIcon(arg.timestamp);
@@ -736,8 +724,8 @@ void ApplicationLauncherIcon::EnsureWindowState()
       // If monitor is -1 (or negative), show on all monitors.
       if (monitor < 0)
       {
-        for (unsigned j; j < monitors::MAX; j++)
-            ++number_of_windows_on_monitor[j];
+        for (unsigned j; j < monitors::MAX; ++j)
+          ++number_of_windows_on_monitor[j];
       }
       else
       {
@@ -746,12 +734,18 @@ void ApplicationLauncherIcon::EnsureWindowState()
     }
   }
 
-  for (unsigned i = 0; i < monitors::MAX; i++)
+  for (unsigned i = 0; i < monitors::MAX; ++i)
     SetNumberOfWindowsVisibleOnMonitor(number_of_windows_on_monitor[i], i);
 
   WindowsChanged.emit();
 }
 
+void ApplicationLauncherIcon::EnsureWindowsLocation()
+{
+  EnsureWindowState();
+  UpdateIconGeometries(GetCenters());
+}
+
 void ApplicationLauncherIcon::UpdateDesktopQuickList()
 {
   std::string const& desktop_file = DesktopFile();
@@ -836,8 +830,6 @@ void ApplicationLauncherIcon::EnsureMenuItemsWindowsReady()
   if (windows.size() < 2)
     return;
 
-  Window active = WindowManager::Default().GetActiveWindow();
-
   // add menu items for all open windows
   for (auto const& w : windows)
   {
@@ -861,7 +853,7 @@ void ApplicationLauncherIcon::EnsureMenuItemsWindowsReady()
         wm.Raise(xid);
     });
 
-    if (xid == active)
+    if (w->active())
     {
       dbusmenu_menuitem_property_set(menu_item, DBUSMENU_MENUITEM_PROP_TOGGLE_TYPE, DBUSMENU_MENUITEM_TOGGLE_RADIO);
       dbusmenu_menuitem_property_set_int(menu_item, DBUSMENU_MENUITEM_PROP_TOGGLE_STATE, DBUSMENU_MENUITEM_TOGGLE_STATE_CHECKED);
@@ -1112,7 +1104,7 @@ AbstractLauncherIcon::MenuItemsVector ApplicationLauncherIcon::GetMenus()
 
 void ApplicationLauncherIcon::UpdateIconGeometries(std::vector<nux::Point3> const& centers)
 {
-  if (app_->type() == "webapp")
+  if (app_->type() == AppType::WEBAPP)
     return;
 
   nux::Geometry geo(0, 0, icon_size, icon_size);
@@ -1168,6 +1160,7 @@ void ApplicationLauncherIcon::OnDndEnter()
     if (!IsRunning())
       return false;
 
+    UBusManager::SendMessage(UBUS_OVERLAY_CLOSE_REQUEST);
     Focus(ActionArg(ActionArg::Source::LAUNCHER, 1, timestamp));
 
     if (GetWindows(WindowFilter::ON_CURRENT_DESKTOP).size() > 1)
@@ -1186,7 +1179,8 @@ bool ApplicationLauncherIcon::IsFileManager()
 {
   auto const& desktop_file = DesktopFile();
 
-  return boost::algorithm::ends_with(desktop_file, "nautilus.desktop") ||
+  return boost::algorithm::ends_with(desktop_file, "org.gnome.Nautilus.desktop") ||
+         boost::algorithm::ends_with(desktop_file, "nautilus.desktop") ||
          boost::algorithm::ends_with(desktop_file, "nautilus-folder-handler.desktop") ||
          boost::algorithm::ends_with(desktop_file, "nautilus-home.desktop");
 }
@@ -1262,14 +1256,14 @@ bool ApplicationLauncherIcon::ShowInSwitcher(bool current)
 
 bool ApplicationLauncherIcon::AllowDetailViewInSwitcher() const
 {
-  return app_->type() != "webapp";
+  return app_->type() != AppType::WEBAPP;
 }
 
 uint64_t ApplicationLauncherIcon::SwitcherPriority()
 {
   uint64_t result = 0;
   // Webapps always go at the back.
-  if (app_->type() == "webapp")
+  if (app_->type() == AppType::WEBAPP)
     return result;
 
   for (auto& window : app_->GetWindows())
