@@ -677,7 +677,9 @@ void Launcher::FillRenderArg(AbstractLauncherIcon::Ptr const& icon,
   // goes for 0.0f when fully unfolded, to 1.0f folded
   float folding_progress = CLAMP((center.y + c_icon_size - folding_threshold) / (float) c_icon_size, 0.0f, 1.0f);
   float unfold_progress = icon->GetQuirkProgress(AbstractLauncherIcon::Quirk::UNFOLDED, monitor());
+  float active_progress = icon->GetQuirkProgress(AbstractLauncherIcon::Quirk::ACTIVE, monitor());
 
+  unfold_progress = CLAMP(unfold_progress + active_progress, 0.0f, 1.0f);
   folding_progress *= 1.0f - unfold_progress;
 
   float half_size = (folded_size / 2.0f) + (c_icon_size / 2.0f - folded_size / 2.0f) * (1.0f - folding_progress);
@@ -686,6 +688,11 @@ void Launcher::FillRenderArg(AbstractLauncherIcon::Ptr const& icon,
   float present_progress = icon->GetQuirkProgress(AbstractLauncherIcon::Quirk::PRESENTED, monitor());
   icon_hide_offset *= 1.0f - (present_progress * icon->PresentUrgency());
 
+  if (present_progress > 0.0f)
+  {
+    parent_->ShowWindow(true);
+  }
+
   // icon is crossing threshold, start folding
   center.z += folded_z_distance * folding_progress;
   arg.rotation.x = animation_neg_rads * folding_progress;
@@ -738,7 +745,7 @@ nux::Color FullySaturateColor(nux::Color color)
 }
 
 void Launcher::RenderArgs(std::list<RenderArg> &launcher_args,
-                          nux::Geometry& box_geo, float* launcher_alpha, nux::Geometry const& parent_abs_geo)
+                          nux::Geometry& box_geo, float* launcher_alpha, nux::Geometry const& parent_abs_geo, bool& force_show_window)
 {
   nux::Geometry const& geo = GetGeometry();
   LauncherModel::iterator it;
@@ -774,9 +781,12 @@ void Launcher::RenderArgs(std::list<RenderArg> &launcher_args,
     sum += height;
 
     // magic constant must some day be explained, for now suffice to say this constant prevents the bottom from "marching";
-    float magic_constant = 1.3f;
+    const float magic_constant = 1.3f;
 
     float unfold_progress = (*it)->GetQuirkProgress(AbstractLauncherIcon::Quirk::UNFOLDED, monitor());
+    float active_progress = (*it)->GetQuirkProgress(AbstractLauncherIcon::Quirk::ACTIVE, monitor());
+
+    unfold_progress = CLAMP(unfold_progress + active_progress, 0.0f, 1.0f);
     folding_threshold -= CLAMP(sum - launcher_height, 0.0f, height * magic_constant) * (folding_constant + (1.0f - folding_constant) * unfold_progress);
   }
 
@@ -912,6 +922,8 @@ void Launcher::RenderArgs(std::list<RenderArg> &launcher_args,
   folding_threshold += shelf_delta;
   center.y += shelf_delta;
 
+  force_show_window = false;
+
   for (it = model_->shelf_begin(); it != model_->shelf_end(); ++it)
   {
     RenderArg arg;
@@ -921,6 +933,9 @@ void Launcher::RenderArgs(std::list<RenderArg> &launcher_args,
                   autohide_offset, folded_z_distance, animation_neg_rads);
     arg.colorify = colorify;
     launcher_args.push_back(arg);
+
+    if (autohide_offset != 0)
+      force_show_window = true;
   }
 }
 
@@ -1108,6 +1123,9 @@ void Launcher::SetHidden(bool hide_launcher)
   postreveal_mousemove_delta_x_ = 0;
   postreveal_mousemove_delta_y_ = 0;
 
+  if (!hide_launcher)
+    parent_->ShowWindow(true);
+
   if (nux::GetWindowThread()->IsEmbeddedWindow())
     parent_->EnableInputWindow(!hide_launcher, launcher::window_title, false, false);
 
@@ -1584,6 +1602,7 @@ void Launcher::SetupIconAnimations(AbstractLauncherIcon::Ptr const& icon)
 {
   icon->SetQuirkDuration(AbstractLauncherIcon::Quirk::VISIBLE, ANIM_DURATION_SHORT, monitor());
   icon->SetQuirkDuration(AbstractLauncherIcon::Quirk::RUNNING, ANIM_DURATION_SHORT, monitor());
+  icon->SetQuirkDuration(AbstractLauncherIcon::Quirk::ACTIVE, ANIM_DURATION_SHORT, monitor());
   icon->SetQuirkDuration(AbstractLauncherIcon::Quirk::STARTING, (ANIM_DURATION_LONG * MAX_STARTING_BLINKS * STARTING_BLINK_LAMBDA * 2), monitor());
   icon->SetQuirkDuration(AbstractLauncherIcon::Quirk::PULSE_ONCE, (ANIM_DURATION_LONG * PULSE_BLINK_LAMBDA * 2), monitor());
   icon->SetQuirkDuration(AbstractLauncherIcon::Quirk::PRESENTED, ANIM_DURATION, monitor());
@@ -1677,10 +1696,19 @@ void Launcher::DrawContent(nux::GraphicsEngine& GfxContext, bool force_draw)
   ROP.SrcBlend = GL_ONE;
   ROP.DstBlend = GL_ONE_MINUS_SRC_ALPHA;
 
+  bool force_show_window;
   nux::Geometry const& geo_absolute = GetAbsoluteGeometry();
-  RenderArgs(args, bkg_box, &launcher_alpha, geo_absolute);
+  RenderArgs(args, bkg_box, &launcher_alpha, geo_absolute, force_show_window);
   bkg_box.width -= RIGHT_LINE_WIDTH.CP(cv_);
 
+  if (options()->hide_mode != LAUNCHER_HIDE_NEVER &&
+      bkg_box.x + bkg_box.width <= 0 &&
+      hide_machine_.reveal_progress <= 0 &&
+      !force_show_window)
+  {
+    parent_->ShowWindow(false);
+  }
+
   nux::Color clear_colour = nux::Color(0x00000000);
 
   // clear region
@@ -2224,9 +2252,10 @@ void Launcher::ScrollLauncher(int wheel_delta)
 
 #ifdef USE_X11
 
-ui::EdgeBarrierSubscriber::Result Launcher::HandleBarrierEvent(ui::PointerBarrierWrapper* owner, ui::BarrierEvent::Ptr event)
+ui::EdgeBarrierSubscriber::Result Launcher::HandleBarrierEvent(ui::PointerBarrierWrapper::Ptr const& owner, ui::BarrierEvent::Ptr event)
 {
-  if (hide_machine_.GetQuirk(LauncherHideMachine::EXTERNAL_DND_ACTIVE))
+  if (hide_machine_.GetQuirk(LauncherHideMachine::EXTERNAL_DND_ACTIVE) ||
+      hide_machine_.GetQuirk(LauncherHideMachine::LOCK_HIDE))
   {
     return ui::EdgeBarrierSubscriber::Result::NEEDS_RELEASE;
   }
@@ -2270,7 +2299,10 @@ ui::EdgeBarrierSubscriber::Result Launcher::HandleBarrierEvent(ui::PointerBarrie
     return ui::EdgeBarrierSubscriber::Result::IGNORED;
 
   if (!owner->IsFirstEvent())
+  {
+    parent_->ShowWindow(true);
     hide_machine_.AddRevealPressure(event->velocity);
+  }
 
   return ui::EdgeBarrierSubscriber::Result::HANDLED;
 }
@@ -2308,13 +2340,17 @@ void Launcher::RecvQuicklistOpened(nux::ObjectPtr<QuicklistView> const& quicklis
 
 void Launcher::RecvQuicklistClosed(nux::ObjectPtr<QuicklistView> const& quicklist)
 {
-  nux::Point pt = nux::GetWindowCompositor().GetMousePosition();
-  if (!GetAbsoluteGeometry().IsInside(pt))
+  if (!IsInKeyNavMode())
   {
-    // The Quicklist just closed and the mouse is outside the launcher.
-    SetHover(false);
-    SetStateMouseOverLauncher(false);
+    nux::Point pt = nux::GetWindowCompositor().GetMousePosition();
+    if (!GetAbsoluteGeometry().IsInside(pt))
+    {
+      // The Quicklist just closed and the mouse is outside the launcher.
+      SetHover(false);
+      SetStateMouseOverLauncher(false);
+    }
   }
+
   // Cancel any prior state that was set before the Quicklist appeared.
   SetActionState(ACTION_NONE);
 
@@ -2346,6 +2382,13 @@ void Launcher::MouseDownLogic(int x, int y, unsigned long button_flags, unsigned
 
   if (launcher_icon)
   {
+    if (IsInKeyNavMode())
+    {
+      key_nav_terminate_request.emit();
+    }
+
+    model_->SetSelection(model_->IconIndex(launcher_icon));
+
     icon_mouse_down_ = launcher_icon;
     // if MouseUp after the time ended -> it's an icon drag, otherwise, it's starting an app
     auto cb_func = sigc::bind(sigc::mem_fun(this, &Launcher::StartIconDragTimeout), x, y);
