@@ -31,6 +31,8 @@
 
 #include "unity-shared/AbstractIconRenderer.h"
 #include "unity-shared/BackgroundEffectHelper.h"
+#include "unity-shared/EMConverter.h"
+#include "unity-shared/RawPixel.h"
 #include "DevicesSettings.h"
 #include "DndData.h"
 #include "unity-shared/Introspectable.h"
@@ -77,6 +79,9 @@ public:
 
   AbstractLauncherIcon::Ptr GetSelectedMenuIcon() const;
 
+  void SetScrollInactiveIcons(bool scroll);
+  void SetLauncherMinimizeWindow(bool click_to_minimize);
+
   void SetIconSize(int tile_size, int icon_size);
   int GetIconSize() const;
 
@@ -147,10 +152,9 @@ public:
   void EnterKeyNavMode();
   void ExitKeyNavMode();
   bool IsInKeyNavMode() const;
-
   bool IsOverlayOpen() const;
 
-  static const int ANIM_DURATION_SHORT;
+  void ClearTooltip();
 
   void RenderIconToTexture(nux::GraphicsEngine&, nux::ObjectPtr<nux::IOpenGLBaseTexture> const&, AbstractLauncherIcon::Ptr const&);
 
@@ -161,7 +165,7 @@ public:
 protected:
   // Introspectable methods
   std::string GetName() const;
-  void AddProperties(GVariantBuilder* builder);
+  void AddProperties(debug::IntrospectionData&);
 
   void ProcessDndEnter();
   void ProcessDndLeave();
@@ -196,10 +200,11 @@ private:
 #endif
 
 #ifdef USE_X11
-  ui::EdgeBarrierSubscriber::Result HandleBarrierEvent(ui::PointerBarrierWrapper* owner, ui::BarrierEvent::Ptr event);
+  ui::EdgeBarrierSubscriber::Result HandleBarrierEvent(ui::PointerBarrierWrapper::Ptr const& owner, ui::BarrierEvent::Ptr event);
 #endif
 
-  void OnPluginStateChanged();
+  void OnExpoChanged();
+  void OnSpreadChanged();
 
   void OnSelectionChanged(AbstractLauncherIcon::Ptr const& selection);
 
@@ -208,8 +213,8 @@ private:
   bool OnScrollTimeout();
 
   void SetUrgentTimer(int urgent_wiggle_period);
-  void WiggleUrgentIcon(AbstractLauncherIcon::Ptr const& icon);
-  void HandleUrgentIcon(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current);
+  void AnimateUrgentIcon(AbstractLauncherIcon::Ptr const& icon);
+  void HandleUrgentIcon(AbstractLauncherIcon::Ptr const& icon);
   bool OnUrgentTimeout();
 
   void SetMousePosition(int x, int y);
@@ -221,9 +226,7 @@ private:
 
   void OnDragWindowAnimCompleted();
 
-  bool IconNeedsAnimation(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
   bool IconDrawEdgeOnly(AbstractLauncherIcon::Ptr const& icon) const;
-  bool AnimationInProgress() const;
 
   void SetActionState(LauncherActionState actionstate);
   LauncherActionState GetActionState() const;
@@ -233,30 +236,22 @@ private:
   bool MouseOverBottomScrollArea();
 
   float DragOutProgress() const;
-  float IconDesatValue(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconPresentProgress(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconUnfoldProgress(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconUrgentProgress(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconShimmerProgress(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconUrgentPulseValue(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconPulseOnceValue(AbstractLauncherIcon::Ptr const& icon, struct timespec const &current) const;
-  float IconUrgentWiggleValue(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconStartingBlinkValue(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconStartingPulseValue(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconBackgroundIntensity(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconProgressBias(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconDropDimValue(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconCenterTransitionProgress(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
-  float IconVisibleProgress(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current) const;
+  float IconUrgentPulseValue(AbstractLauncherIcon::Ptr const& icon) const;
+  float IconPulseOnceValue(AbstractLauncherIcon::Ptr const& icon) const;
+  float IconUrgentWiggleValue(AbstractLauncherIcon::Ptr const& icon) const;
+  float IconStartingBlinkValue(AbstractLauncherIcon::Ptr const& icon) const;
+  float IconStartingPulseValue(AbstractLauncherIcon::Ptr const& icon) const;
+  float IconBackgroundIntensity(AbstractLauncherIcon::Ptr const& icon) const;
+  float IconProgressBias(AbstractLauncherIcon::Ptr const& icon) const;
 
   void SetHidden(bool hidden);
 
   void UpdateChangeInMousePosition(int delta_x, int delta_y);
 
-  void  SetDndDelta(float x, float y, nux::Geometry const& geo, timespec const& current);
+  void  SetDndDelta(float x, float y, nux::Geometry const& geo);
   float DragLimiter(float x);
 
-  void SetupRenderArg(AbstractLauncherIcon::Ptr const& icon, struct timespec const& current, ui::RenderArg& arg);
+  void SetupRenderArg(AbstractLauncherIcon::Ptr const& icon, ui::RenderArg& arg);
   void FillRenderArg(AbstractLauncherIcon::Ptr const& icon,
                      ui::RenderArg& arg,
                      nux::Point3& center,
@@ -266,14 +261,15 @@ private:
                      float folded_spacing,
                      float autohide_offset,
                      float folded_z_distance,
-                     float animation_neg_rads,
-                     struct timespec const& current);
+                     float animation_neg_rads);
 
   void RenderArgs(std::list<ui::RenderArg> &launcher_args,
                   nux::Geometry& box_geo, float* launcher_alpha, nux::Geometry const& parent_abs_geo);
 
   void OnIconAdded(AbstractLauncherIcon::Ptr const& icon);
   void OnIconRemoved(AbstractLauncherIcon::Ptr const& icon);
+  void OnIconNeedsRedraw(AbstractLauncherIcon::Ptr const& icon, int monitor);
+  void SetupIconAnimations(AbstractLauncherIcon::Ptr const& icon);
 
   void OnTooltipVisible(nux::ObjectPtr<nux::View> view);
 
@@ -312,10 +308,13 @@ private:
   void DndHoveredIconReset();
   bool DndIsSpecialRequest(std::string const& uri) const;
 
+  void OnDPIChanged();
+
   LauncherModel::Ptr model_;
   MockableBaseWindow* parent_;
   ui::AbstractIconRenderer::Ptr icon_renderer_;
   nux::ObjectPtr<nux::View> active_tooltip_;
+  std::set<AbstractLauncherIcon::Ptr> animating_urgent_icons_;
 
   // used by keyboard/a11y-navigation
   AbstractLauncherIcon::Ptr icon_under_mouse_;
@@ -337,7 +336,7 @@ private:
 
   LauncherActionState launcher_action_state_;
 
-  int icon_size_;
+  RawPixel icon_size_;
   int dnd_delta_y_;
   int dnd_delta_x_;
   int postreveal_mousemove_delta_x_;
@@ -348,9 +347,7 @@ private:
   int enter_y_;
   int last_button_press_;
   int drag_icon_position_;
-  int urgent_wiggle_time_;
-  bool urgent_acked_;
-  bool urgent_timer_running_;
+  int urgent_animation_period_;
   bool urgent_ack_needed_;
   float drag_out_delta_x_;
   bool drag_gesture_ongoing_;
@@ -364,7 +361,6 @@ private:
 
   unity::DndData dnd_data_;
   nux::DndAction drag_action_;
-  struct timespec urgent_finished_time_;
 
   BaseTexturePtr launcher_sheen_;
   BaseTexturePtr launcher_pressure_effect_;
@@ -380,6 +376,7 @@ private:
 
   UBusManager ubus_;
   glib::SourceManager sources_;
+  EMConverter::Ptr cv_;
 
   friend class TestLauncher;
 };
