@@ -21,7 +21,7 @@
 #include <Nux/Nux.h>
 #include <NuxCore/Logger.h>
 #include <UnityCore/ConnectionManager.h>
-#include <UnityCore/GTKWrapper.h>
+#include <UnityCore/GLibWrapper.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
@@ -59,6 +59,7 @@ PanelIndicatorEntryView::PanelIndicatorEntryView(Entry::Ptr const& proxy, int pa
   , cv_(unity::Settings::Instance().em(monitor_))
 {
   proxy_->active_changed.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::OnActiveChanged));
+  proxy_->show_now_changed.connect(sigc::mem_fun(&show_now_changed, &sigc::signal<void, bool>::emit));
   proxy_->updated.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::Refresh));
 
   InputArea::mouse_down.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::OnMouseDown));
@@ -84,11 +85,6 @@ PanelIndicatorEntryView::PanelIndicatorEntryView(Entry::Ptr const& proxy, int pa
   Refresh();
 }
 
-PanelIndicatorEntryView::~PanelIndicatorEntryView()
-{
-  // Nothing to do...
-}
-
 void PanelIndicatorEntryView::OnActiveChanged(bool is_active)
 {
   active_changed.emit(this, is_active);
@@ -114,10 +110,16 @@ void PanelIndicatorEntryView::ShowMenu(int button)
     });
 
     wm.TerminateExpo();
+    return;
   }
 
   if (wm.IsScaleActive())
+  {
+    if (type_ == MENU)
+      return;
+
     wm.TerminateScale();
+  }
 
   auto const& abs_geo = GetAbsoluteGeometry();
   proxy_->ShowMenu(abs_geo.x, abs_geo.y + abs_geo.height, button);
@@ -222,8 +224,8 @@ glib::Object<GdkPixbuf> PanelIndicatorEntryView::MakePixbuf(int size)
     case GTK_IMAGE_GICON:
     {
       GtkIconTheme* theme = gtk_icon_theme_get_default();
-      auto flags = static_cast<GtkIconLookupFlags>(0);
-      gtk::IconInfo info;
+      auto flags = GTK_ICON_LOOKUP_FORCE_SIZE;
+      glib::Object<GtkIconInfo> info;
 
       if (image_type == GTK_IMAGE_GICON)
       {
@@ -426,7 +428,7 @@ void PanelIndicatorEntryView::Refresh()
       }
     }
 
-    glib::Object<PangoContext> context(gdk_pango_context_get_for_screen(gdk_screen_get_default()));
+    glib::Object<PangoContext> context(gdk_pango_context_get());
     std::shared_ptr<PangoFontDescription> desc(pango_font_description_from_string(font.c_str()), pango_font_description_free);
     pango_context_set_font_description(context, desc.get());
     pango_context_set_language(context, gtk_get_default_language());
