@@ -54,8 +54,8 @@
 #include "DashController.h"
 #include "UnitySettings.h"
 #include "DashStyle.h"
+#include "EdgeBarrierController.h"
 #include "FavoriteStoreGSettings.h"
-#include "FontSettings.h"
 #include "ShortcutController.h"
 #include "LauncherController.h"
 #include "LockScreenController.h"
@@ -239,7 +239,7 @@ public:
 
   switcher::Controller::Ptr switcher_controller();
   launcher::Controller::Ptr launcher_controller();
-  std::shared_ptr<lockscreen::Controller> lockscreen_controller();
+  lockscreen::Controller::Ptr lockscreen_controller();
 
   bool DoesPointIntersectUnityGeos(nux::Point const& pt);
 
@@ -326,13 +326,13 @@ private:
   Settings unity_settings_;
   dash::Style    dash_style_;
   panel::Style   panel_style_;
-  FontSettings   font_settings_;
   internal::FavoriteStoreGSettings favorite_store_;
   ThumbnailGenerator thumbnail_generator_;
   lockscreen::Settings lockscreen_settings_;
 
   /* The window thread should be the last thing removed, as c++ does it in reverse order */
   std::unique_ptr<nux::WindowThread> wt;
+  WindowManager& WM;
 
   menu::Manager::Ptr menus_;
   std::shared_ptr<decoration::Manager> deco_manager_;
@@ -348,7 +348,8 @@ private:
   session::DBusManager::Ptr session_dbus_manager_;
   session::Controller::Ptr  session_controller_;
   lockscreen::DBusManager::Ptr screensaver_dbus_manager_;
-  std::shared_ptr<lockscreen::Controller> lockscreen_controller_;
+  lockscreen::Controller::Ptr lockscreen_controller_;
+  ui::EdgeBarrierController::Ptr edge_barriers_;
   debug::DebugDBusInterface debugger_;
   std::unique_ptr<BGHash>   bghash_;
   spread::Filter::Ptr       spread_filter_;
@@ -368,7 +369,7 @@ private:
   typedef std::vector<CompActionPtr> ShortcutActions;
   ShortcutActions _shortcut_actions;
   std::map<CancelActionTarget, CompActionPtr> _escape_actions;
-  std::map<int, unsigned int> windows_for_monitor_;
+  std::unordered_map<int, unsigned int> windows_for_monitor_;
 
   /* keyboard-nav mode */
   CompWindow* newFocusedWindow;
@@ -423,6 +424,7 @@ private:
   UBusManager ubus_manager_;
   glib::SourceManager sources_;
   connection::Wrapper hud_ungrab_slot_;
+  connection::Manager launcher_size_connections_;
 
   CompRegion buffered_compiz_damage_this_frame_;
   CompRegion buffered_compiz_damage_last_frame_;
