@@ -158,6 +158,10 @@ class UnityPanel(UnityIntrospectionObject, KeybindingsHelper):
     def title(self):
         return self.menus.panel_title
 
+    @property
+    def focused(self):
+        return self.menus.focused
+
     @property
     def desktop_is_active(self):
         return self.menus.desktop_active
