@@ -144,18 +144,39 @@ class DashRevealTests(DashTestCase):
 
         char_win = self.process_manager.start_app("Character Map")
         self.keybinding("window/maximize")
-        self.process_manager.start_app("Calculator")
+        calc_win = self.process_manager.start_app("Calculator")
 
         self.unity.dash.ensure_visible()
 
-        #Click bottom right of the screen
-        w = self.display.get_screen_width()
-        h = self.display.get_screen_height()
+        # Click bottom right of the screen, but take into account the non-maximized window -
+        # we do not want to click on it as it focuses the wrong window
+        w = self.display.get_screen_width() - 1
+        h = self.display.get_screen_height() - 1
+
+        # If the mouse is over the non-maximized window, move it away from it.
+        (calc_x, calc_y, calc_w, calc_h) = calc_win.get_windows()[0].geometry
+        if calc_x <= w <= calc_x+calc_w:
+            grab_padding = 15
+            w = w - (calc_w + grab_padding)
+
         self.mouse.move(w,h)
         self.mouse.click()
 
         self.assertProperty(char_win, is_active=True)
 
+    def test_dash_opens_when_fullscreen_window(self):
+        """ The Dash must not open if a window is fullscreen. """
+        gedit = self.process_manager.start_app("Text Editor")
+        self.keyboard.press_and_release('F11')
+
+        monitor = gedit.get_windows()[0].monitor
+        move_mouse_to_screen(monitor)
+
+        self.keybinding("dash/reveal")
+        self.addCleanup(self.unity.dash.ensure_hidden)
+
+        self.assertThat(self.unity.dash.visible, Eventually(Equals(True)))
+
 
 class DashRevealWithSpreadTests(DashTestCase):
     """Test the interaction of the Dash with the Spead/Scale
@@ -310,7 +331,7 @@ class DashKeyNavTests(DashTestCase):
         self.assertThat(scopebar.active_scope, Eventually(Equals(focused_icon)))
 
         # scopebar should lose focus after activation.
-        self.assertThat(scopebar.focused_scope_icon, Eventually(Equals("")))
+        self.assertFalse(hasattr(scopebar, 'focused_scope_icon'))
 
     def test_focus_returns_to_searchbar(self):
         """This test makes sure that the focus is returned to the searchbar of the newly
@@ -326,7 +347,7 @@ class DashKeyNavTests(DashTestCase):
         self.keyboard.press_and_release("Enter")
 
         self.assertThat(scopebar.active_scope, Eventually(Equals(focused_icon)))
-        self.assertThat(scopebar.focused_scope_icon, Eventually(Equals("")))
+        self.assertFalse(hasattr(scopebar, 'focused_scope_icon'))
 
         # Now we make sure if the newly activated scope searchbar have the focus.
         self.keyboard.type("HasFocus")
@@ -789,7 +810,9 @@ class PreviewInvocationTests(DashTestCase):
         scope = self.unity.dash.reveal_application_scope()
         self.addCleanup(self.unity.dash.ensure_hidden)
 
-        # wait for "More suggestions" category
+        self.keyboard.type("Software Updater")
+
+        # wait for "Installed" category
         category = self.wait_for_category(scope, _("Installed"))
 
         # wait for some results
@@ -1364,3 +1387,17 @@ class DashCrossMonitorsTests(DashTestCase):
             self.mouse.click()
 
             self.assertThat(self.unity.dash.visible, Eventually(Equals(False)))
+
+    def test_dash_opens_on_second_monitor_if_first_has_fullscreen_window(self):
+        """ The Dash must open if the mouse is over the second monitor while the
+            first monitor has a fullscreen window. """
+
+        gedit = self.process_manager.start_app("Text Editor")
+        monitor = gedit.get_windows()[0].monitor
+        self.keyboard.press_and_release('F11')
+
+        move_mouse_to_screen((monitor + 1) % self.display.get_num_screens())
+        self.keybinding("dash/reveal")
+        self.addCleanup(self.unity.dash.ensure_hidden)
+
+        self.assertThat(self.unity.dash.visible, Eventually(Equals(True)))
