@@ -1,5 +1,5 @@
 /*
- * Copyright 2012 Canonical Ltd.
+ * Copyright 2012,2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3, as published
@@ -21,9 +21,9 @@
 #include <Nux/Nux.h>
 #include "PanelMenuView.h"
 #include "PanelStyle.h"
-#include "UnitySettings.h"
 #include "UBusMessages.h"
-#include "StandaloneWindowManager.h"
+#include "mock_menu_manager.h"
+#include "test_standalone_wm.h"
 #include "test_uscreen_mock.h"
 #include "test_utils.h"
 
@@ -31,21 +31,40 @@ using namespace testing;
 
 namespace unity
 {
+namespace panel
+{
 
 struct TestPanelMenuView : public testing::Test
 {
   TestPanelMenuView()
-    : WM(dynamic_cast<StandaloneWindowManager*>(&WindowManager::Default()))
+    : menu_manager(std::make_shared<menu::MockManager>())
+    , menu_view(menu_manager)
   {}
 
   struct MockPanelMenuView : public PanelMenuView
   {
+    MockPanelMenuView(menu::Manager::Ptr const& menu_manager)
+      : PanelMenuView(menu_manager)
+    {
+      view_opened_signal_.Disconnect();
+      active_win_changed_signal_.Disconnect();
+      active_app_changed_signal_.Disconnect();
+      view_closed_signal_.Disconnect();
+      maximized_wins_.clear();
+    }
+
     MOCK_METHOD0(QueueDraw, void());
     MOCK_CONST_METHOD1(GetActiveViewName, std::string(bool));
 
     using PanelMenuView::GetCurrentTitle;
+    using PanelMenuView::ShouldDrawButtons;
+    using PanelMenuView::ShouldDrawMenus;
+    using PanelMenuView::FindAreaUnderMouse;
     using PanelMenuView::window_buttons_;
     using PanelMenuView::titlebar_grab_area_;
+    using PanelMenuView::we_control_active_;
+    using PanelMenuView::spread_showing_;
+    using PanelMenuView::maximized_wins_;
   };
 
   nux::ObjectPtr<nux::BaseWindow> AddPanelToWindow(int monitor)
@@ -53,7 +72,7 @@ struct TestPanelMenuView : public testing::Test
     nux::ObjectPtr<nux::BaseWindow> panel_win(new nux::BaseWindow());
     auto const& monitor_geo = uscreen.GetMonitorGeometry(monitor);
     panel_win->SetGeometry(monitor_geo);
-    panel_win->SetMaximumHeight(panelStyle.panel_height());
+    panel_win->SetMaximumHeight(panelStyle.PanelHeight(monitor));
     panel_win->SetLayout(new nux::HLayout(NUX_TRACKER_LOCATION));
     panel_win->GetLayout()->AddView(&menu_view, 1);
     panel_win->GetLayout()->SetContentDistribution(nux::MAJOR_POSITION_START);
@@ -62,6 +81,7 @@ struct TestPanelMenuView : public testing::Test
     panel_win->ComputeContentSize();
 
     menu_view.SetMonitor(monitor);
+    menu_view.maximized_wins_.clear();
 
     return panel_win;
   }
@@ -70,29 +90,34 @@ protected:
   // The order is important, i.e. menu_view needs
   // panel::Style that needs Settings
   MockUScreen uscreen;
-  Settings settings;
   panel::Style panelStyle;
-  StandaloneWindowManager* WM;
+  testwrapper::StandaloneWM WM;
+  menu::MockManager::Ptr menu_manager;
   testing::NiceMock<MockPanelMenuView> menu_view;
 };
 
+TEST_F(TestPanelMenuView, Construction)
+{
+  EXPECT_FALSE(menu_view.spread_showing_);
+  EXPECT_FALSE(menu_view.GetControlsActive());
+  ASSERT_TRUE(menu_view.window_buttons_.IsValid());
+}
+
 TEST_F(TestPanelMenuView, Escaping)
 {
+  menu_manager->integrated_menus = false;
   ON_CALL(menu_view, GetActiveViewName(testing::_)).WillByDefault(Return("<>'"));
-  static const char *escapedText = "Panel d&amp;Inici";
-  EXPECT_TRUE(menu_view.GetCurrentTitle().empty());
+  auto escapedText = "Panel d'Inici";
+  ASSERT_TRUE(menu_view.GetCurrentTitle().empty());
 
   UBusManager ubus;
   ubus.SendMessage(UBUS_LAUNCHER_START_KEY_NAV);
-  ubus.SendMessage(UBUS_LAUNCHER_SELECTION_CHANGED,
-                   g_variant_new_string(escapedText));
-  Utils::WaitUntilMSec([this] {return menu_view.GetCurrentTitle() == escapedText;});
+  ubus.SendMessage(UBUS_LAUNCHER_SELECTION_CHANGED, glib::Variant(escapedText));
+  Utils::WaitUntilMSec([this, &escapedText] {return menu_view.GetCurrentTitle() == escapedText;});
 
-
-  WM->SetScaleActive(true);
-  WM->SetScaleActiveForGroup(true);
+  menu_view.we_control_active_ = true;
   ubus.SendMessage(UBUS_LAUNCHER_END_KEY_NAV);
-  Utils::WaitUntilMSec([this] {return menu_view.GetCurrentTitle() == "&lt;&gt;&apos;";});
+  Utils::WaitUntilMSec([this] {return menu_view.GetCurrentTitle() == "<>'";});
 }
 
 TEST_F(TestPanelMenuView, QueuesDrawOnButtonsOpacityChange)
@@ -101,12 +126,35 @@ TEST_F(TestPanelMenuView, QueuesDrawOnButtonsOpacityChange)
   menu_view.window_buttons_->opacity.changed.emit(0.5f);
 }
 
+TEST_F(TestPanelMenuView, SpreadActivation)
+{
+  EXPECT_CALL(menu_view, QueueDraw());
+  WM->SetScaleActive(true);
+  EXPECT_TRUE(menu_view.spread_showing_);
+}
+
+TEST_F(TestPanelMenuView, ShouldDrawButtonsOnSpread)
+{
+  WM->SetScaleActive(true);
+  EXPECT_TRUE(menu_view.ShouldDrawButtons());
+}
+
+TEST_F(TestPanelMenuView, ShouldDrawMenusOnSpread)
+{
+  WM->SetScaleActive(true);
+  EXPECT_FALSE(menu_view.ShouldDrawMenus());
+}
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-variable"
+
 struct ProgressTester : TestPanelMenuView, WithParamInterface<double> {};
 INSTANTIATE_TEST_CASE_P(TestPanelMenuView, ProgressTester, Range(0.0, 1.0, 0.1));
 
 TEST_P(ProgressTester, RestoreOnGrabInBiggerWorkArea)
 {
   uscreen.SetupFakeMultiMonitor();
+  connection::Manager conn;
   unsigned monitor = uscreen.GetMonitors().size() - 1;
   auto const& monitor_geo = uscreen.GetMonitorGeometry(monitor);
   WM->SetWorkareaGeometry(monitor_geo);
@@ -140,5 +188,7 @@ TEST_P(ProgressTester, RestoreOnGrabInBiggerWorkArea)
   EXPECT_EQ(max_window->geo(), expected_geo);
 }
 
+#pragma GCC diagnostic pop
 
-}
+} // panel namespace
+} // unity namespace
