@@ -41,29 +41,6 @@ namespace
 
 #define DEFAULT_GICON ". GThemedIcon cmake"
 
-GdkPixbuf* GetIconData(std::string icon_hint, int size)
-{
-  GdkPixbuf *pbuf;
-  GtkIconTheme *theme;
-  glib::Error error;
-
-  theme = gtk_icon_theme_get_default();
-  glib::Object<GIcon> icon(g_icon_new_for_string(icon_hint.c_str(), NULL));
-
-  if (icon.IsType(G_TYPE_ICON))
-  {
-    gtk::IconInfo info(gtk_icon_theme_lookup_by_gicon(theme, icon, size, (GtkIconLookupFlags)0));
-    pbuf = gtk_icon_info_load_icon(info, &error);
-
-    if (error)
-    {
-      pbuf = NULL;
-    }
-  }
-
-  return pbuf;
-}
-
 } // namespace [anonymous]
 
 class TestResultRenderer : public testing::Test
