@@ -171,9 +171,7 @@ TEST_F(TestSwitcherController, ShowSwitcherNoShowDeskop)
 {
   EXPECT_CALL(*mock_window_, ShowWindow(true, _)).Times(AtLeast(1));
 
-  controller_->SetShowDesktopDisabled(true);
-  ASSERT_TRUE(controller_->IsShowDesktopDisabled());
-
+  controller_->show_desktop_disabled = true;
   controller_->Show(ShowMode::ALL, SortMode::LAUNCHER_ORDER, icons_);
   Utils::WaitUntilMSec([this] { return controller_->Visible(); });
   ASSERT_TRUE(controller_->StartIndex() == 0);
@@ -187,7 +185,7 @@ TEST_F(TestSwitcherController, ShowSwitcherNoShowDeskop)
 
 TEST_F(TestSwitcherController, ShowSwitcherNoResults)
 {
-  controller_->SetShowDesktopDisabled(true);
+  controller_->show_desktop_disabled = true;
   std::vector<unity::launcher::AbstractLauncherIcon::Ptr> results;
   EXPECT_CALL(*mock_window_, ShowWindow(true, _)).Times(0);
 
