@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 Canonical Ltd.
+ * Copyright 2013,2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3, as published
@@ -117,8 +117,7 @@ TEST_F(TestSwitcherView, Initiate)
   EXPECT_EQ(switcher.vertical_size, switcher.tile_size + VERTICAL_PADDING * 2);
   EXPECT_EQ(switcher.text_size, 15);
   EXPECT_EQ(switcher.animation_length, 250);
-  EXPECT_EQ(switcher.monitor, -1);
-  EXPECT_EQ(switcher.spread_size, 3.5f);
+  EXPECT_EQ(switcher.monitor, 0);
   ASSERT_NE(switcher.text_view_, nullptr);
   ASSERT_NE(switcher.icon_renderer_, nullptr);
   EXPECT_EQ(switcher.icon_renderer_->pip_style, ui::OVER_TILE);
@@ -151,6 +150,8 @@ TEST_F(TestSwitcherView, SkipAnimation)
   EXPECT_DOUBLE_EQ(switcher.GetCurrentProgress(), 1.0f);
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-variable"
 
 struct AnimationProgress : TestSwitcherView, testing::WithParamInterface<float> {};
 INSTANTIATE_TEST_CASE_P(TestSwitcherView, AnimationProgress, testing::Range<float>(0.0, 1.0, 0.1));
@@ -217,5 +218,7 @@ TEST_P(AnimationProgress, ResizeRenderTargets)
   }
 }
 
+#pragma GCC diagnostic pop
+
 }
 }
