@@ -35,6 +35,7 @@
 #include <UnityCore/GTKWrapper.h>
 
 #include "unity-shared/Timer.h"
+#include "unity-shared/UnitySettings.h"
 
 namespace unity
 {
@@ -233,7 +234,7 @@ private:
       else if (icon.IsType(G_TYPE_ICON))
       {
         GtkIconInfo *info = ::gtk_icon_theme_lookup_by_gicon(impl->theme_, icon, size,
-                                                             static_cast<GtkIconLookupFlags>(0));
+                                                             static_cast<GtkIconLookupFlags>(GTK_ICON_LOOKUP_FORCE_SIZE));
         if (info)
         {
           icon_info = info;
@@ -313,10 +314,8 @@ private:
         PangoContext* pango_context = NULL;
         GdkScreen* screen = gdk_screen_get_default(); // not ref'ed
         glib::String font;
-        int dpi = -1;
 
         g_object_get(gtk_settings_get_default(), "gtk-font-name", &font, NULL);
-        g_object_get(gtk_settings_get_default(), "gtk-xft-dpi", &dpi, NULL);
         cairo_set_font_options(cr, gdk_screen_get_font_options(screen));
         layout = pango_cairo_create_layout(cr);
         std::shared_ptr<PangoFontDescription> desc(pango_font_description_from_string(font), pango_font_description_free);
@@ -345,10 +344,8 @@ private:
         pango_layout_set_markup(layout, escaped_text, -1);
 
         pango_context = pango_layout_get_context(layout);  // is not ref'ed
-        pango_cairo_context_set_font_options(pango_context,
-                                             gdk_screen_get_font_options(screen));
-        pango_cairo_context_set_resolution(pango_context,
-                                           dpi == -1 ? 96.0f : dpi/(float) PANGO_SCALE);
+        pango_cairo_context_set_font_options(pango_context, gdk_screen_get_font_options(screen));
+        pango_cairo_context_set_resolution(pango_context, 96.0 * Settings::Instance().font_scaling());
         pango_layout_context_changed(layout);
 
         // find proper font size
@@ -802,7 +799,7 @@ private:
   bool CoalesceTasksCb();
 
 private:
-  std::map<std::string, glib::Object<GdkPixbuf>> cache_;
+  std::unordered_map<std::string, glib::Object<GdkPixbuf>> cache_;
   /* FIXME: the reference counting of IconLoaderTasks with shared pointers
    * is currently somewhat broken, and the queued_tasks_ member is what keeps
    * it from crashing randomly.
@@ -810,7 +807,7 @@ private:
    * tasks, but when they are being completed in a worker thread, the thread
    * should own them as well (yet it doesn't), this could cause trouble
    * in the future... You've been warned! */
-  std::map<std::string, IconLoaderTask::Ptr> queued_tasks_;
+  std::unordered_map<std::string, IconLoaderTask::Ptr> queued_tasks_;
   std::queue<IconLoaderTask::Ptr> tasks_;
   std::unordered_map<Handle, IconLoaderTask::Ptr> task_map_;
   std::vector<IconLoaderTask*> finished_tasks_;
@@ -830,7 +827,7 @@ IconLoader::Impl::Impl()
   , theme_(::gtk_icon_theme_get_default())
   , handle_counter_(0)
 {
-  theme_changed_signal_.Connect(theme_, "changed", [&] (GtkIconTheme*) {
+  theme_changed_signal_.Connect(theme_, "changed", [this] (GtkIconTheme*) {
     /* Since the theme has been changed we can clear the cache, however we
      * could include two improvements here:
      *  1) clear only the themed icons in cache
@@ -923,11 +920,7 @@ void IconLoader::Impl::CalculateTextHeight(int* width, int* height)
   cairo_t* cr = util_cg.GetInternalContext();
 
   glib::String font;
-  int dpi = 0;
-  g_object_get(settings,
-                 "gtk-font-name", &font,
-                 "gtk-xft-dpi", &dpi,
-                 NULL);
+  g_object_get(settings, "gtk-font-name", &font, nullptr);
   std::shared_ptr<PangoFontDescription> desc(pango_font_description_from_string(font), pango_font_description_free);
   pango_font_description_set_weight(desc.get(), PANGO_WEIGHT_BOLD);
   pango_font_description_set_size(desc.get(), FONT_SIZE * PANGO_SCALE);
@@ -939,7 +932,7 @@ void IconLoader::Impl::CalculateTextHeight(int* width, int* height)
   PangoContext* cxt = pango_layout_get_context(layout);
   GdkScreen* screen = gdk_screen_get_default();
   pango_cairo_context_set_font_options(cxt, gdk_screen_get_font_options(screen));
-  pango_cairo_context_set_resolution(cxt, dpi / (double) PANGO_SCALE);
+  pango_cairo_context_set_resolution(cxt, 96.0 * Settings::Instance().font_scaling());
   pango_layout_context_changed(layout);
 
   PangoRectangle log_rect;
