@@ -55,9 +55,18 @@ PluginAdapter& PluginAdapter::Default()
 }
 
 /* static */
-void PluginAdapter::Initialize(CompScreen* screen)
+PluginAdapter& PluginAdapter::Initialize(CompScreen* screen)
 {
-  global_instance.reset(new PluginAdapter(screen));
+  if (!global_instance)
+  {
+    global_instance.reset(new PluginAdapter(screen));
+  }
+  else
+  {
+    LOG_ERROR(logger) << "Already Initialized!";
+  }
+
+  return *global_instance;
 }
 
 PluginAdapter::PluginAdapter(CompScreen* screen)
@@ -136,6 +145,16 @@ void PluginAdapter::NotifyStateChange(CompWindow* window, unsigned int state, un
   {
     window_restored.emit(window->id());
   }
+
+  if ((state & CompWindowStateFullscreenMask) == CompWindowStateFullscreenMask)
+  {
+    window_fullscreen.emit(window->id());
+  }
+  else if (((last_state & CompWindowStateFullscreenMask) == CompWindowStateFullscreenMask) &&
+           !((state & CompWindowStateFullscreenMask) == CompWindowStateFullscreenMask))
+  {
+    window_unfullscreen.emit(window->id());
+  }
 }
 
 void PluginAdapter::Notify(CompWindow* window, CompWindowNotify notify)
@@ -521,6 +540,14 @@ bool PluginAdapter::IsWindowHorizontallyMaximized(Window window_id) const
   return false;
 }
 
+bool PluginAdapter::IsWindowFullscreen(Window window_id) const
+{
+  if (CompWindow* window = m_Screen->findWindow(window_id))
+    return ((window->state() & CompWindowStateFullscreenMask) == CompWindowStateFullscreenMask);
+
+  return false;
+}
+
 bool PluginAdapter::HasWindowDecorations(Window window_id) const
 {
   return compiz_utils::IsWindowFullyDecorable(m_Screen->findWindow(window_id));
@@ -554,9 +581,10 @@ bool PluginAdapter::IsWindowOnCurrentDesktop(Window window_id) const
 
 bool PluginAdapter::IsWindowObscured(Window window_id) const
 {
-  CompWindow* window = m_Screen->findWindow(window_id);
+  if (_spread_state)
+    return false;
 
-  if (window)
+  if (CompWindow* window = m_Screen->findWindow(window_id))
   {
     if (window->inShowDesktopMode())
       return true;
@@ -626,7 +654,7 @@ Window PluginAdapter::GetTopMostValidWindowInViewport() const
     if (window->defaultViewport() == screen_vp &&
         window->isViewable() && window->isMapped() &&
         !window->minimized() && !window->inShowDesktopMode() &&
-        !(window->state() & CompWindowStateAboveMask) &&
+        !((window->state() & CompWindowStateAboveMask) && !window->focused()) &&
         !(window->type() & CompWindowTypeSplashMask) &&
         !(window->type() & CompWindowTypeDockMask) &&
         !window->overrideRedirect() &&
@@ -771,7 +799,10 @@ void PluginAdapter::UnMinimize(Window window_id)
 {
   CompWindow* window = m_Screen->findWindow(window_id);
   if (window && (window->actions() & CompWindowActionMinimizeMask))
+  {
     window->unminimize();
+    window->show();
+  }
 }
 
 void PluginAdapter::Shade(Window window_id)
@@ -952,6 +983,7 @@ void PluginAdapter::FocusWindowGroup(std::vector<Window> const& window_ids,
       if (forced_unminimize)
         {
           top_window->unminimize();
+          top_window->show();
         }
 
       top_window->raise();
@@ -1454,6 +1486,40 @@ void PluginAdapter::OnWindowClosed(CompWindow *w)
     _last_focused_window = NULL;
 }
 
+void PluginAdapter::UnmapAllNoNuxWindowsSync()
+{
+  for (auto const& window : m_Screen->windows())
+  {
+    if (!IsNuxWindow(window) && (window->isMapped() || window->isViewable()))
+    {
+      if (window->overrideRedirect())
+      {
+        XUnmapWindow(m_Screen->dpy(), window->id());
+      }
+      else
+      {
+        window->hide();
+      }
+    }
+  }
+
+  XSync(m_Screen->dpy(), False);
+}
+
+bool PluginAdapter::IsNuxWindow(CompWindow* value)
+{
+  std::vector<Window> const& xwns = nux::XInputWindow::NativeHandleList();
+  auto id = value->id();
+
+  unsigned int size = xwns.size();
+  for (unsigned int i = 0; i < size; ++i)
+  {
+    if (xwns[i] == id)
+      return true;
+  }
+  return false;
+}
+
 void PluginAdapter::AddProperties(debug::IntrospectionData& wrapper)
 {
   wrapper.add(GetScreenGeometry())
