@@ -50,7 +50,7 @@ private:
   CompAction* GetAction(std::string const& name) const;
 
   CompAction* primary_action_;
-  std::map<std::string, CompAction*> actions_;
+  std::unordered_map<std::string, CompAction*> actions_;
 };
 
 
@@ -60,7 +60,7 @@ public:
   // You shouldn't get the PluginAdapter if you really want a WindowManager.
   // The PluginAdapter::Default should really only be called from within unityshell plugin.
   static PluginAdapter& Default();
-  static void Initialize(CompScreen* screen);
+  static PluginAdapter& Initialize(CompScreen* screen);
 
   nux::Property<bool> bias_active_to_viewport;
 
@@ -122,6 +122,7 @@ public:
   bool IsWindowMaximized(Window window_id) const;
   bool IsWindowVerticallyMaximized(Window window_id) const;
   bool IsWindowHorizontallyMaximized(Window window_id) const;
+  bool IsWindowFullscreen(Window window_id) const;
   bool IsWindowDecorated(Window window_id) const;
   bool IsWindowOnCurrentDesktop(Window window_id) const;
   bool IsWindowObscured(Window window_id) const;
@@ -193,6 +194,9 @@ public:
 
   Window GetTopWindowAbove(Window xid) const;
 
+  void UnmapAllNoNuxWindowsSync();
+  static bool IsNuxWindow(CompWindow* value);
+
 protected:
   PluginAdapter(CompScreen* screen);
   void AddProperties(debug::IntrospectionData&);
