@@ -26,6 +26,7 @@
 #include <UnityCore/GLibSource.h>
 
 #include "unity-shared/Introspectable.h"
+#include "unity-shared/ExpanderView.h"
 
 namespace nux
 {
@@ -63,6 +64,7 @@ public:
   nux::Property<bool> can_refine_search;
   nux::ROProperty<bool> im_active;
   nux::ROProperty<bool> im_preedit;
+  nux::ROProperty<bool> in_live_search;
   nux::Property<unsigned> live_search_wait;
   nux::Property<double> scale;
 
@@ -88,10 +90,7 @@ private:
   bool OnLiveSearchTimeout();
   bool OnSpinnerStartCb();
 
-  std::string get_search_string() const;
   bool set_search_string(std::string const& string);
-  bool get_im_active() const;
-  bool get_im_preedit() const;
   bool show_filter_hint_;
 
   std::string GetName() const;
@@ -110,7 +109,7 @@ private:
   StaticCairoText* hint_;
   nux::LinearLayout* expander_layout_;
   IMTextEntry* pango_entry_;
-  nux::View* expander_view_;
+  ExpanderView* expander_view_;
   nux::HLayout* filter_layout_;
   StaticCairoText* show_filters_;
   nux::VLayout* arrow_layout_;
