@@ -160,9 +160,12 @@ public:
   virtual bool RestoreInputFocus();
 
   virtual std::string GetWindowName(Window window_id) const;
+  virtual bool IsOnscreenKeyboard(Window window_id) const;
   virtual std::string GetStringProperty(Window window_id, Atom) const;
   virtual std::vector<long> GetCardinalProperty(Window window_id, Atom) const;
 
+  virtual void UnmapAllNoNuxWindowsSync() {};
+
   // Mock functions
   StandaloneWindow::Ptr GetWindowByXid(Window window_id) const;
   void AddStandaloneWindow(StandaloneWindow::Ptr const& window);
